/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
`include "sim_common_definitions.vh"

`ifdef _BUILD_BRD_ARTY_XC7A35T
`define _BUILD_BRD_ARTY_XC7A35T_100T
`elsif _BUILD_BRD_ARTY_XC7A100T
`define _BUILD_BRD_ARTY_XC7A35T_100T
`endif

`ifdef _BUILD_BRD_ARTY_XC7A35T_100T
`define NO_OF_CHANNELS 9
`elsif _BUILD_BRD_ARTY_XC7S50T
`define NO_OF_CHANNELS 9
`elsif _BUILD_BRD_ARTY_XC7Z020
`define NO_OF_CHANNELS 9
`endif

module sim_adc_in_ports
(
input clk,

`ifdef _BUILD_BRD_ARTY_XC7A35T_100T
input vaux_p4, vaux_n4, vaux_p5, vaux_n5, vaux_p6, vaux_n6, vaux_p7, vaux_n7, vaux_p15, vaux_n15, vaux_p0, vaux_n0, vaux_p12, vaux_n12, vaux_p13, vaux_n13, vaux_p14, vaux_n14,
output reg [15:0] CH_A0, reg [15:0] CH_A1, reg [15:0] CH_A2, reg [15:0] CH_A3, reg [15:0] CH_A4, reg [15:0] CH_A5, reg [15:0] CH_A6A7, reg [15:0] CH_A8A9, reg [15:0] CH_A10A11
`elsif _BUILD_BRD_ARTY_XC7S50T
input vaux_p0, vaux_n0, vaux_p1, vaux_n1, vaux_p9, vaux_n9, vaux_p2, vaux_n2, vaux_p10, vaux_n10, vaux_p11, vaux_n11, vaux_p8, vaux_n8, vaux_p3, vaux_n3, vaux_vp, vaux_vn,
output reg [15:0] CH_A0, reg [15:0] CH_A1, reg [15:0] CH_A2, reg [15:0] CH_A3, reg [15:0] CH_A4, reg [15:0] CH_A5, reg [15:0] CH_A6A7, reg [15:0] CH_A8A9, reg [15:0] CH_VPVN
`elsif _BUILD_BRD_ARTY_XC7Z020
input vaux_p1, vaux_n1, vaux_p9, vaux_n9, vaux_p6, vaux_n6, vaux_p15, vaux_n15, vaux_p5, vaux_n5, vaux_p13, vaux_n13, vaux_p12, vaux_n12, vaux_p0, vaux_n0, vaux_p8, vaux_n8,
output reg [15:0] CH_A0, reg [15:0] CH_A1, reg [15:0] CH_A2, reg [15:0] CH_A3, reg [15:0] CH_A4, reg [15:0] CH_A5, reg [15:0] CH_A6A7, reg [15:0] CH_A8A9, reg [15:0] CH_A10A11
`endif

);

wire enable;  
wire ready;
reg ready_prev;
wire [15:0] adc_data;

reg [3:0] cur_ch_idx;
reg [6:0] ch_addressa [`NO_OF_CHANNELS-1:0];

//xadc instantiation. connect the eoc_out .den_in to get continuous conversion
xadc_wiz_0 xadc
(
    .daddr_in(ch_addressa[cur_ch_idx]),            // Address bus for the dynamic reconfiguration port
    .dclk_in(clk),          // Clock input for the dynamic reconfiguration port
    .den_in(enable),        // Enable Signal for the dynamic reconfiguration port
    .di_in(0),              // Input data bus for the dynamic reconfiguration port
    .dwe_in(0),             // Write Enable for the dynamic reconfiguration port
    .reset_in(0),           // Reset signal for the System Monitor control logic
    .busy_out(),            // ADC Busy signal
    .channel_out(),         // Channel Selection Outputs
    .do_out(adc_data),      // Output data bus for dynamic reconfiguration port
    .eoc_out(enable),       // End of Conversion Signal
    .eos_out(),             // End of Sequence Signal
    .alarm_out(),           // OR'ed output of all the Alarms  
    .drdy_out(ready),       // Data ready signal for the dynamic reconfiguration port
`ifdef _BUILD_BRD_ARTY_XC7A35T_100T
    .vauxp4(vaux_p4),
    .vauxn4(vaux_n4),
    .vauxp5(vaux_p5),
    .vauxn5(vaux_n5),
    .vauxp6(vaux_p6),
    .vauxn6(vaux_n6),
    .vauxp7(vaux_p7),
    .vauxn7(vaux_n7),
    .vauxp15(vaux_p15),
    .vauxn15(vaux_n15),
    .vauxp0(vaux_p0),
    .vauxn0(vaux_n0),
    .vauxp12(vaux_p12),
    .vauxn12(vaux_n12),
    .vauxp13(vaux_p13),
    .vauxn13(vaux_n13),
    .vauxp14(vaux_p14),
    .vauxn14(vaux_n14)
`elsif _BUILD_BRD_ARTY_XC7S50T
    .vp_in(vaux_vp),
    .vn_in(vaux_vn),
    .vauxp0(vaux_p0),
    .vauxn0(vaux_n0),
    .vauxp1(vaux_p1),
    .vauxn1(vaux_n1),
    .vauxp2(vaux_p2),
    .vauxn2(vaux_n2),
    .vauxp3(vaux_p3),
    .vauxn3(vaux_n3),
    .vauxp8(vaux_p8),
    .vauxn8(vaux_n8),
    .vauxp9(vaux_p9),
    .vauxn9(vaux_n9),
    .vauxp10(vaux_p10),
    .vauxn10(vaux_n10),
    .vauxp11(vaux_p11),
    .vauxn11(vaux_n11)
`elsif _BUILD_BRD_ARTY_XC7Z020
    .vauxp0(vaux_p0),
    .vauxn0(vaux_n0),
    .vauxp1(vaux_p1),
    .vauxn1(vaux_n1),
    .vauxp5(vaux_p5),
    .vauxn5(vaux_n5),
    .vauxp6(vaux_p6),
    .vauxn6(vaux_n6),
    .vauxp8(vaux_p8),
    .vauxn8(vaux_n8),
    .vauxp9(vaux_p9),
    .vauxn9(vaux_n9),
    .vauxp12(vaux_p12),
    .vauxn12(vaux_n12),
    .vauxp13(vaux_p13),
    .vauxn13(vaux_n13),
    .vauxp15(vaux_p15),
    .vauxn15(vaux_n15)
`endif
);

initial begin
    ready_prev = 0;
`ifdef _BUILD_BRD_ARTY_XC7A35T_100T
    ch_addressa[0] = 8'h14; //A0
    ch_addressa[1] = 8'h15; //A1
    ch_addressa[2] = 8'h16; //A2
    ch_addressa[3] = 8'h17; //A3
    ch_addressa[4] = 8'h1F; //A4
    ch_addressa[5] = 8'h10; //A5
    ch_addressa[6] = 8'h1C; //A6 - A7 differential
    ch_addressa[7] = 8'h1D; //A8 - A9 differential
    ch_addressa[8] = 8'h1E; //A10 - A11 differential
    CH_A0 <= 0; CH_A1 <= 0; CH_A2 <= 0; CH_A3 <= 0; CH_A4 <= 0; CH_A5 <= 0; CH_A6A7 <= 0; CH_A8A9 <= 0; CH_A10A11 <= 0;
`elsif _BUILD_BRD_ARTY_XC7S50T   
    ch_addressa[0] = 8'h10; //A0
    ch_addressa[1] = 8'h11; //A1
    ch_addressa[2] = 8'h19; //A2
    ch_addressa[3] = 8'h12; //A3
    ch_addressa[4] = 8'h1A; //A4
    ch_addressa[5] = 8'h1B; //A5
    ch_addressa[6] = 8'h18; //A6 - A7 differential
    ch_addressa[7] = 8'h13; //A8 - A9 differential
    ch_addressa[8] = 8'h03; //Dedicated V diferential
    CH_A0 <= 0; CH_A1 <= 0; CH_A2 <= 0; CH_A3 <= 0; CH_A4 <= 0; CH_A5 <= 0; CH_A6A7 <= 0; CH_A8A9 <= 0; CH_VPVN <= 0;
`elsif _BUILD_BRD_ARTY_XC7Z020
    ch_addressa[0] = 8'h11; //A0
    ch_addressa[1] = 8'h19; //A1
    ch_addressa[2] = 8'h16; //A2
    ch_addressa[3] = 8'h1F; //A3
    ch_addressa[4] = 8'h15; //A4
    ch_addressa[5] = 8'h1D; //A5
    ch_addressa[6] = 8'h1C; //A6 - A7 differential
    ch_addressa[7] = 8'h10; //A8 - A9 differential
    ch_addressa[8] = 8'h18; //A10 - A11 differential
    CH_A0 <= 0; CH_A1 <= 0; CH_A2 <= 0; CH_A3 <= 0; CH_A4 <= 0; CH_A5 <= 0; CH_A6A7 <= 0; CH_A8A9 <= 0; CH_A10A11 <= 0;
`endif
    cur_ch_idx = 0;
end

//choose next channel
always @(posedge clk)
begin
  if(ready == 1 && ready_prev == 0)
  begin
 `ifdef _BUILD_BRD_ARTY_XC7A35T_100T
    if(cur_ch_idx == 0) CH_A0 <= adc_data;
    else if(cur_ch_idx == 1) CH_A1 = adc_data;
    else if(cur_ch_idx == 2) CH_A2 = adc_data;
    else if(cur_ch_idx == 3) CH_A3 = adc_data;
    else if(cur_ch_idx == 4) CH_A4 = adc_data;
    else if(cur_ch_idx == 5) CH_A5 = adc_data;
    else if(cur_ch_idx == 6) CH_A6A7 = adc_data;
    else if(cur_ch_idx == 7) CH_A8A9 = adc_data;
    else if(cur_ch_idx == 8) CH_A10A11 = adc_data;
`elsif _BUILD_BRD_ARTY_XC7S50T   
    if(cur_ch_idx == 0) CH_A0 <= adc_data;
    else if(cur_ch_idx == 1) CH_A1 = adc_data;
    else if(cur_ch_idx == 2) CH_A2 = adc_data;
    else if(cur_ch_idx == 3) CH_A3 = adc_data;
    else if(cur_ch_idx == 4) CH_A4 = adc_data;
    else if(cur_ch_idx == 5) CH_A5 = adc_data;
    else if(cur_ch_idx == 6) CH_A6A7 = adc_data;
    else if(cur_ch_idx == 7) CH_A8A9 = adc_data;
    else if(cur_ch_idx == 8) CH_VPVN = adc_data;
`elsif _BUILD_BRD_ARTY_XC7Z020
    if(cur_ch_idx == 0) CH_A0 <= adc_data;
    else if(cur_ch_idx == 1) CH_A1 = adc_data;
    else if(cur_ch_idx == 2) CH_A2 = adc_data;
    else if(cur_ch_idx == 3) CH_A3 = adc_data;
    else if(cur_ch_idx == 4) CH_A4 = adc_data;
    else if(cur_ch_idx == 5) CH_A5 = adc_data;
    else if(cur_ch_idx == 6) CH_A6A7 = adc_data;
    else if(cur_ch_idx == 7) CH_A8A9 = adc_data;
    else if(cur_ch_idx == 8) CH_A10A11 = adc_data;
`endif
    //
    cur_ch_idx = cur_ch_idx + 1;
    if(cur_ch_idx == `NO_OF_CHANNELS)
        cur_ch_idx = 0;
  end
  ready_prev <= ready;
end   

endmodule
/*
module sim_adc_in_ports
(
input clk,
input we,
input [3:0] waddr,
input A0_P, A0_N, A1_P, A1_N, A2_P, A2_N, A3_P, A3_N, A4_P, A4_N, A5_P, A5_N, A6_P, A7_N, A8_P, A9_N, A10_P, A11_N, AV_P, AV_N,
output reg [31:0] data_out
);

wire enable;  
wire ready;
reg ready_d1;
wire ready_rising;
wire [15:0] adc_data;   

reg [3:0] cur_ch_idx;
reg [6:0] ch_addressa [`NO_OF_CHANNELS-1:0];
reg [31:0] ain_buffer[`MAX_AIN_PORTS-1:0];

//xadc instantiation. connect the eoc_out .den_in to get continuous conversion
xadc_wiz_0 xadc
(
    .daddr_in(ch_addressa[cur_ch_idx]),            // Address bus for the dynamic reconfiguration port
    .dclk_in(clk),             // Clock input for the dynamic reconfiguration port
    .den_in(enable),              // Enable Signal for the dynamic reconfiguration port
    .di_in(0),               // Input data bus for the dynamic reconfiguration port
    .dwe_in(0),              // Write Enable for the dynamic reconfiguration port
    .reset_in(0),            // Reset signal for the System Monitor control logic
    .busy_out(),            // ADC Busy signal
    .channel_out(),         // Channel Selection Outputs
    .do_out(adc_data),              // Output data bus for dynamic reconfiguration port
    .eoc_out(enable),             // End of Conversion Signal
    .eos_out(),             // End of Sequence Signal
    .alarm_out(),           // OR'ed output of all the Alarms  
    .drdy_out(ready),            // Data ready signal for the dynamic reconfiguration port
    
    .vp_in(AV_P),                        // input wire vp_in
    .vn_in(AV_N),                        // input wire vn_in
//    .vauxp0(A0_P),                      // input wire vauxp0
//    .vauxn0(A0_N),                      // input wire vauxn0
//    .vauxp1(A1_P),                      // input wire vauxp1
//    .vauxn1(A1_N),                      // input wire vauxn1
//    .vauxp2(A3_P),                      // input wire vauxp2
//    .vauxn2(A3_N),                      // input wire vauxn2
//    .vauxp3(A8_P),                      // input wire vauxp3
//    .vauxn3(A9_N),                      // input wire vauxn3
//    .vauxp8(A6_P),                      // input wire vauxp8
//    .vauxn8(A7_N),                      // input wire vauxn8
//    .vauxp9(A2_P),                      // input wire vauxp9
//    .vauxn9(A2_N),                      // input wire vauxn9
//    .vauxp10(A4_P),                    // input wire vauxp10
//    .vauxn10(A4_N),                    // input wire vauxn10
//    .vauxp11(A5_P),                    // input wire vauxp11
//    .vauxn11(A5_N)                     // input wire vauxn11
    
    .vauxp4(A0_P),
    .vauxn4(A0_N),
    .vauxp5(A1_P),
    .vauxn5(A1_N),
    .vauxp6(A2_P),
    .vauxn6(A2_N),
    .vauxp7(A3_P),
    .vauxn7(A3_N),
    .vauxp15(A4_P),
    .vauxn15(A4_N),
    .vauxp0(A5_P),
    .vauxn0(A5_N),
    .vauxp12(A6_P),
    .vauxn12(A7_N),
    .vauxp13(A8_P),
    .vauxn13(A9_N),
    .vauxp14(A10_P),
    .vauxn14(A11_N)
);

initial begin
    ready_d1 = 0;
//    ch_addressa[0] = 8'h10; //A0
//    ch_addressa[1] = 8'h11; //A1
//    ch_addressa[2] = 8'h19; //A2
//    ch_addressa[3] = 8'h12; //A3
//    ch_addressa[4] = 8'h1A; //A4
//    ch_addressa[5] = 8'h1B; //A5
//    ch_addressa[6] = 8'h18; //A6 - A7 differential
//    ch_addressa[7] = 8'h13; //A8 - A9 differential
//    ch_addressa[8] = 8'h03; //Dedicated V diferential
    ch_addressa[0] = 8'h14; //A0
    ch_addressa[1] = 8'h15; //A1
    ch_addressa[2] = 8'h16; //A2
    ch_addressa[3] = 8'h17; //A3
    ch_addressa[4] = 8'h1F; //A4
    ch_addressa[5] = 8'h10; //A5
    ch_addressa[6] = 8'h1C; //A6 - A7 differential
    ch_addressa[7] = 8'h1D; //A8 - A9 differential
    ch_addressa[8] = 8'h1E; //A10 - A11 differential
    //
    data_out = 0;
    for(cur_ch_idx=0; cur_ch_idx<`MAX_AIN_PORTS-1; cur_ch_idx=cur_ch_idx+1)
        ain_buffer[cur_ch_idx] <= 0;
    cur_ch_idx = 0;
    ain_buffer[`MAX_AIN_PORTS-1] <= 0;
end

always @(posedge clk)
begin
  ready_d1 <= ready;
end
assign ready_rising = ready && !ready_d1 ? 1'b1 : 1'b0;

//choose next channel
always @(posedge clk)
begin
  if(ready_rising == 1)
  begin
    ain_buffer[cur_ch_idx] = adc_data;
    if(cur_ch_idx == `NO_OF_CHANNELS-1)
        cur_ch_idx = 0;
    else
        cur_ch_idx = cur_ch_idx + 1;
  end
end   

//add to out buffer
always @ (posedge clk)
begin
    if(we) begin
        data_out <= ain_buffer[waddr];
    end
    else data_out <= data_out; 
end

endmodule
*/
