/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "eth_comm.h"
#include "sim_helper.h"
#include "iodevice.h"

static volatile _UINT32 g_eth_write_idx = 0;
static volatile bool g_system_initialized;

void eth_comm(volatile _UINT32 *eth_buf, volatile _UINT32 eth_shm[ETH_SHM_SIZE])
{
#pragma HLS INTERFACE m_axi depth=8192 port=eth_buf offset=off name=eth_buf
#pragma HLS INTERFACE bram port=eth_shm
#pragma HLS RESOURCE variable=eth_shm core=RAM_1P_BRAM
#pragma HLS RESET variable=g_system_initialized

	if(g_system_initialized == 0)
	{
		g_eth_write_idx = 0;
		//
	    //bitSet(eth_buf[0x07FC>>2], 4);//transmit control reg.: 4-enable loop back
	    //bitSet(eth_buf[0x07FC>>2], 3);//write interrupt
	    //bitSet(eth_buf[0x17FC>>2], 3);//recv. control reg.: 3-enable recv. interrupt
	    //bitSet(eth_buf[0x07F8>>2], 31);//gei
	    //set mac
	    //val = (0x00 << 24) | (0x00 << 16) | (0x5E << 8) | (0x00 << 0); eth_buf[0] = val;
	    //val = (0xFA << 24) | (0xCE << 16) | (0x00 << 8) | (0x00 << 0); eth_buf[1] = val;
	    //_UINT32 tcr = eth_buf[0x07FC>>2];
	    //bitSet(tcr, 1);//set program bit
	    //bitSet(tcr, 0);//set data valid
	    //eth_buf[0x07FC>>2] = tcr;
	    //set destination and source address
		_UINT32 val = 0;
		val = (0xff << 0) | (0xff << 8) | (0xff << 16) | (0xff << 24); eth_buf[0] = val;
	    val = (0xff << 0) | (0xff << 8) | (0x00 << 16) | (0x00 << 24); eth_buf[1] = val;
	    val = (0x5E << 0) | (0x00 << 8) | (0xFA << 16) | (0xCE << 24); eth_buf[2] = val;
	}
	else
	{
		_UINT32 in_val = 0;
		_UINT32 out_val = 0;
		_UINT32 rcr = eth_buf[0x17FC>>2];
		if(bitRead(rcr, 0) == 1)//available
		{
			in_val = eth_buf[(0x1000>>2)+3];
			_BYTE bval = (in_val >> 16);//start byte
			if(bval == TARGETHW_COMM_START_BYTE)
			{
                //get size (in bytes)
				_UINT32 recvd_size = 0;
                bval = (in_val >> 24); recvd_size |= bval;
				in_val = eth_buf[(0x1000>>2)+4];
				bval = (in_val >> 0); recvd_size |= (((_UINT32)bval) << 8);
                //update shm
				out_val = TARGETHW_COMM_START_BYTE;//start byte
				out_val |= ((recvd_size << 8) & 0x00ffff00);//size (16 bit)
                bval = (in_val >> 8); out_val |= (bval << 24);//check sum
                eth_shm[0] = out_val;
				bval = (in_val >> 16); out_val = bval;
				bval = (in_val >> 24); out_val |= (bval << 8);
                //
                recvd_size += TARGETHW_COMM_HEADER_SIZE;
                recvd_size = (recvd_size >> 2) & 0xffff;//by 4
				for(_UINT32 i=0; i<recvd_size-1; ++i)//for(_UINT32 i=1; i<recvd_size; ++i) - this logic is not working ???
				{
					_UINT32 val = eth_buf[(0x1000>>2)+5+i];
					out_val |= ((val << 16) & 0xffff0000);
					eth_shm[i+1] = out_val;
					out_val = ((val >> 16) & 0xffff);
				}
				eth_shm[ETH_CTRL_RSIZE_IDX] = recvd_size;//in integers
			}
			bitClear(rcr, 0);//set ready to recv. data
			eth_buf[0x17FC>>2] = rcr;
		}
		else
		{
			_UINT32 send_size = eth_shm[ETH_CTRL_WSIZE_IDX];//in integers (not bytes)
			 if(send_size > 0)
			 {
				_UINT32 tcr = eth_buf[0x07FC>>2];
				if(bitRead(tcr, 0) == 0)//ready
				{
					//set destination mac
					if(eth_buf[0] == 0xffffffff)
					{
						_BYTE mac[8];
						in_val = eth_buf[(0x1000>>2)+1];
						mac[0] = in_val >> 16; mac[1] = in_val >> 24;
						in_val = eth_buf[(0x1000>>2)+2];
						mac[2] = in_val >> 0; mac[3] = in_val >> 8; mac[4] = in_val >> 16; mac[5] = in_val >> 24;
						in_val = eth_buf[1];
						mac[6] = in_val >> 16; mac[7] = in_val >> 24;
						eth_buf[0] = mac[0] << 0 | mac[1] << 8 | mac[2] << 16 | mac[3] << 24;
						eth_buf[1] = mac[4] << 0 | mac[5] << 8 | mac[6] << 16 | mac[7] << 24;
					}
					//packet type & data
					_UINT32 in_val = eth_shm[0];
					out_val = (0xca << 0) | (0x59 << 8) | (in_val << 16);//higher 2 bytes are data
					eth_buf[3] = out_val;
					//data
					for(_UINT32 i=0; i<send_size; ++i)
					{
						out_val = (in_val >> 16) & 0xffff;
						in_val = eth_shm[i+1];
						out_val |= ((in_val << 16) & 0xffff0000);
						eth_buf[4+i] = out_val;
					}
					//
					eth_buf[0x07F4 >> 2] = 14+(send_size<<2);//data length in bytes
					bitSet(tcr, 0);//set data valid to send
					eth_buf[0x07FC>>2] = tcr;
					//
					eth_shm[ETH_CTRL_WSIZE_IDX] = 0;
				}
			 }
		}//else
	}//else
	g_system_initialized = 1;
}

