/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "uart_comm.h"
#include "sim_helper.h"
#include "iodevice.h"

static volatile _UINT32 g_rint_data, g_tint_data;
static volatile _BYTE g_rint_data_idx, g_tint_data_idx;
static volatile _UINT32 g_uart_write_idx = 0;
static volatile bool g_system_initialized;

void uart_comm(volatile _UINT32 *uart_buf, volatile _UINT32 uart_shm[UART_SHM_SIZE])
{
#pragma HLS INTERFACE m_axi depth=8192 port=uart_buf offset=off name=uart_buf
#pragma HLS INTERFACE bram port=uart_shm
#pragma HLS RESOURCE variable=uart_shm core=RAM_1P_BRAM
#pragma HLS RESET variable=g_system_initialized

	if(g_system_initialized == 0)
	{
		g_uart_write_idx = 0;
		g_rint_data = g_tint_data = 0;
		g_rint_data_idx = 0;
		g_tint_data_idx = 32;
		//
		_UINT32 ctrl_reg = 0;
		bitSet(ctrl_reg, 0);//reset tx fifo
		bitSet(ctrl_reg, 1);//reset rx fifo
		//bitSet(ctrl_reg, 4);//enable interrupt
		uart_buf[0x0C >> 2] = ctrl_reg;
	}
	else
	{
		_UINT32 status_reg = uart_buf[0x08 >> 2];
		if(bitRead(status_reg, 0))//rx valid
		{
			_UINT32 rbyte = 0;
			if(bitRead(status_reg, 1))//rx full
			{//A
				rbyte = uart_buf[0] & 0xff;
				g_rint_data |= (rbyte << g_rint_data_idx); g_rint_data_idx += 8;
				if(g_rint_data_idx == 32)
				{
					_UINT32 ridx = uart_shm[UART_CTRL_RSIZE_IDX];
					if(ridx >= (UART_SHM_SIZE-UART_CTRL_SIZE)) ridx = 0;
					uart_shm[ridx] = g_rint_data;
					uart_shm[UART_CTRL_RSIZE_IDX] = ridx+1;//recvd_size indicates integers
					//
					g_rint_data = 0;
					g_rint_data_idx = 0;
				}
			}
			//repeat A
			{
				rbyte = uart_buf[0] & 0xff;
				g_rint_data |= (rbyte << g_rint_data_idx); g_rint_data_idx += 8;
				if(g_rint_data_idx == 32)
				{
					_UINT32 ridx = uart_shm[UART_CTRL_RSIZE_IDX];
					if(ridx >= (UART_SHM_SIZE-UART_CTRL_SIZE)) ridx = 0;
					uart_shm[ridx] = g_rint_data;
					uart_shm[UART_CTRL_RSIZE_IDX] = ridx+1;//recvd_size indicates integers
					//
					g_rint_data = 0;
					g_rint_data_idx = 0;
				}
			}
		}
		else
		{
			if(g_tint_data_idx == 32)
			{
				_UINT32 send_size = uart_shm[UART_CTRL_WSIZE_IDX];//send_size in integers
				if(g_uart_write_idx < send_size)
				{
					g_tint_data = uart_shm[g_uart_write_idx];
					g_uart_write_idx++;
					g_tint_data_idx = 0;
				}
				else
				{
					uart_shm[UART_CTRL_WSIZE_IDX] = 0;
					g_uart_write_idx = 0;
					send_size = 0;
					g_tint_data_idx = 32;
				}
			}
			if(g_tint_data_idx < 32)
			{
				_UINT32 status_reg = uart_buf[0x08 >> 2];
				if(bitRead(status_reg, 2) == 1)//tx empty
				{
					uart_buf[0x04 >> 2] = ((g_tint_data >> g_tint_data_idx) & 0xff); g_tint_data_idx += 8;
					uart_buf[0x04 >> 2] = ((g_tint_data >> g_tint_data_idx) & 0xff); g_tint_data_idx += 8;
				}
			}
		}
	}
	g_system_initialized = 1;
}
