/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "fpga.h"
#include "sim_helper.h"
#ifndef UNDER_TESTING
#include "simcode.h"
#include "simcode_global.h"
#include "RootSignalClass.h"
#endif

static volatile bool g_system_initialized;
static volatile _UINT64 g_rt_clk_ticks;

#ifdef UNDER_TESTING
#ifdef FPGA_ENABLE_PCIE_COMM
#define COMM_CTRL_SIZE          PCIE_CTRL_SIZE
#define COMM_DATA_SIZE          PCIE_SHM_SIZE-COMM_CTRL_SIZE
#define COMM_BUF_SIZE           PCIE_SHM_SIZE
#elif defined FPGA_ENABLE_ETH_COMM
#define COMM_CTRL_SIZE          ETH_CTRL_SIZE
#define COMM_DATA_SIZE          ETH_SHM_SIZE-COMM_CTRL_SIZE
#define COMM_BUF_SIZE           ETH_SHM_SIZE
#elif defined FPGA_ENABLE_UART_COMM
#define COMM_CTRL_SIZE          UART_CTRL_SIZE
#define COMM_DATA_SIZE          UART_SHM_SIZE-COMM_CTRL_SIZE
#define COMM_BUF_SIZE           UART_SHM_SIZE
#elif defined FPGA_ENABLE_SHM_0
#define COMM_CTRL_SIZE          SHM0_CTRL_SIZE
#define COMM_DATA_SIZE          SHM0_DATA_SIZE
#define COMM_BUF_SIZE           COMM_DATA_SIZE+COMM_CTRL_SIZE
#endif
#define COMM_CTRL_RSIZE_IDX (COMM_BUF_SIZE-1)//for master
#define COMM_CTRL_WSIZE_IDX (COMM_BUF_SIZE-2)//for master
//
static AOUT_DTYPE a_o = 0;
static _BIT d_o = 0;
//
void setup(volatile _UINT32 *shm);
void loop(volatile _UINT32 *shm, AOUT_DTYPE &ao_LED1_B, AOUT_DTYPE &ao_LED1_G, AOUT_DTYPE &ao_LED1_R, AOUT_DTYPE &ao_LED0_B, AOUT_DTYPE &ao_LED0_G, AOUT_DTYPE &ao_LED0_R, _BIT &do_LED4);
void comm_init(volatile _UINT32 *shm);
_UINT32 comm_available(volatile _UINT32 *shm);
_INT32 comm_peek(_UINT32 idx, volatile _UINT32 *shm);//idx is not validated
void comm_peek_array(volatile _INT32 *buffer, _UINT32 size_to_read, volatile _UINT32 *shm);//no validations are done on size_to_read. starts from 0 index
void comm_read_clear(volatile _UINT32 *shm);
bool comm_write_idle(volatile _UINT32 *shm);
void comm_write(_INT32 data, _UINT32 offset_idx, volatile _UINT32 *shm);
void comm_write_array(volatile _INT32 *data, _UINT32 size, _UINT32 offset_idx, volatile _UINT32 *shm);
void comm_write_flush(_UINT32 wsize, volatile _UINT32 *shm);
void comm_write_clear(volatile _UINT32 *shm);
//
static _INT32 g_local_read_buf[24];
static _UINT16 g_write_size = 0;
#else
static volatile GLOBAL_RUN_PARAM_STRUCT g_run_para;
static RootSignalClass g_rootsig_obj;
#endif

void fpga_main(_UINT64 rt_clk_ticks
#ifdef FPGA_ENABLE_PCIE_COMM
        , volatile _UINT64 pcie_shm[PCIE_SHM_SIZE]
#endif
#ifdef FPGA_ENABLE_ETH_COMM
        , volatile _UINT32 eth_shm[ETH_SHM_SIZE]
#endif
#ifdef FPGA_ENABLE_UART_COMM
        , volatile _UINT32 uart_shm[UART_SHM_SIZE]
#endif
#ifdef FPGA_ENABLE_SHM_0
        , volatile _UINT32 shm0[SHM0_BUF_SIZE]
#endif
#ifdef FPGA_ENABLE_SHM_1
        , volatile _UINT32 shm1[SHM1_BUF_SIZE]
#endif
#ifdef FPGA_ENABLE_SHM_2
        , volatile _UINT32 shm2[SHM2_BUF_SIZE]
#endif
#ifdef FPGA_ENABLE_SHM_3
        , volatile _UINT32 shm3[SHM3_BUF_SIZE]
#endif
        , AIN_DTYPE ai_CH_A1, AIN_DTYPE ai_CH_A0, AIN_DTYPE ai_CH_A2, _BIT di_SW0, _BIT di_BTN2, _BIT di_BTN1, _BIT di_BTN0, _BIT di_SW1, _BIT di_SW2, AOUT_DTYPE &ao_LED1_B, AOUT_DTYPE &ao_LED1_G, AOUT_DTYPE &ao_LED1_R, AOUT_DTYPE &ao_LED0_B, AOUT_DTYPE &ao_LED0_G, AOUT_DTYPE &ao_LED0_R, _BIT &do_LED4
        )
{
//#pragma HLS PIPELINE II=1
//#pragma HLS DATAFLOW
#pragma HLS RESET variable=g_system_initialized
#ifdef FPGA_ENABLE_ETH_COMM
#pragma HLS INTERFACE bram port=eth_shm
#pragma HLS RESOURCE variable=eth_shm core=RAM_1P_BRAM
#endif
//
#ifdef FPGA_ENABLE_UART_COMM
#pragma HLS INTERFACE bram port=uart_shm
#pragma HLS RESOURCE variable=uart_shm core=RAM_1P_BRAM
#endif
//
#ifdef FPGA_ENABLE_SHM_0
#ifdef FPGA_SHM0_AXI_MASTER
#pragma HLS INTERFACE m_axi depth=4196 port=shm0 offset=off
#elif defined FPGA_SHM0_AXI_LITE_SLAVE
#pragma HLS INTERFACE s_axilite port=shm0
#else
#pragma HLS INTERFACE bram port=shm0
#pragma HLS RESOURCE variable=shm0 core=RAM_1P_BRAM
#endif
#endif
//
#ifdef FPGA_ENABLE_SHM_1
#ifdef FPGA_SHM1_AXI_MASTER
#pragma HLS INTERFACE m_axi depth=4196 port=shm1 offset=off
#elif defined FPGA_SHM1_AXI_LITE_SLAVE
#pragma HLS INTERFACE s_axilite port=shm1
#else
#pragma HLS INTERFACE bram port=shm1
#pragma HLS RESOURCE variable=shm1 core=RAM_1P_BRAM
#endif
#endif
//
#ifdef FPGA_ENABLE_SHM_2
#ifdef FPGA_SHM2_AXI_MASTER
#pragma HLS INTERFACE m_axi depth=4196 port=shm2 offset=off
#elif defined FPGA_SHM2_AXI_LITE_SLAVE
#pragma HLS INTERFACE s_axilite port=shm2
#else
#pragma HLS INTERFACE bram port=shm2
#pragma HLS RESOURCE variable=shm2 core=RAM_1P_BRAM
#endif
#endif
//
#ifdef FPGA_ENABLE_SHM_3
#ifdef FPGA_SHM3_AXI_MASTER
#pragma HLS INTERFACE m_axi depth=4196 port=shm3 offset=off
#elif defined FPGA_SHM3_AXI_LITE_SLAVE
#pragma HLS INTERFACE s_axilite port=shm3
#else
#pragma HLS INTERFACE bram port=shm3
#pragma HLS RESOURCE variable=shm3 core=RAM_1P_BRAM
#endif
#endif
	//
        _UINT64 hw_time = micros();
	g_rt_clk_ticks = rt_clk_ticks;
	if(g_system_initialized == 0)
	{
#ifdef UNDER_TESTING
#ifdef FPGA_ENABLE_ETH_COMM
			setup(eth_shm);
#elif defined FPGA_ENABLE_UART_COMM
			setup(uart_shm);
#elif defined FPGA_ENABLE_SHM_0
			setup(shm0);
#endif
#else
            g_run_para.rt_clk_tick = hw_time;
            g_run_para.end_time = 0;
            g_run_para.sim_stepsize = SIM_TIME_STEP;
            g_run_para.plot_stepsize = SIM_PLOT_STEP-1;//to enable > instead of ==
            g_run_para.max_nsamples = 0;
            //
            g_run_para.time_step_0 = 0;
            g_run_para.time_step_1 = 0;
            g_run_para.time_step_plot = SIM_PLOT_STEP+1;
            g_run_para.pause_simulation = 0;
            g_run_para.stop_simulation = 0;
            g_run_para.user_paused = false;
            //
#ifdef _SIM_REAL_TIME
            g_run_para.prev_hw_time = 0;//hw_time;//let first step excute immediatly
#endif
            BLOCK_CONSTRUCTOR_PARAM_STRUCT cpara;
            cpara.nport_data = 0;
            cpara.blk_id = 0;
            g_rootsig_obj.Initialize(&cpara);
            for(int i=0; i<PRERUN_METHOD_CALL_COUNT; ++i)//number of passes
                g_rootsig_obj.PreRun(&g_run_para, uart_shm, di_BTN0, ao_LED0_R, di_BTN1, ao_LED0_G, di_BTN2, ao_LED0_B, do_LED4, ao_LED1_B, ao_LED1_G, ao_LED1_R, ai_CH_A2, ai_CH_A0, ai_CH_A1, di_SW0, di_SW1, di_SW2);
#endif
	}
        else if(g_run_para.user_paused == false)
	{
#ifdef UNDER_TESTING
		//ao0 = ai0;
		//do0 = di0;
#ifdef FPGA_ENABLE_ETH_COMM
		loop(eth_shm, ao0, do0);
#elif defined FPGA_ENABLE_UART_COMM
		loop(uart_shm, ao0, do0);
#elif defined FPGA_ENABLE_SHM_0
		loop(shm0, ao0, do0);
#endif//FPGA_ENABLE_ETH_COMM
#else//UNDER_TESTING
            g_run_para.rt_clk_tick = hw_time;
#if (SIM_TARGETHW_RUN_TIME > 0)
            if(g_run_para.rt_clk_tick > ((SIM_TARGETHW_RUN_TIME*1000)+1000000))
            {
                g_rootsig_obj.PostRun(&g_run_para, uart_shm, di_BTN0, ao_LED0_R, di_BTN1, ao_LED0_G, di_BTN2, ao_LED0_B, do_LED4, ao_LED1_B, ao_LED1_G, ao_LED1_R, ai_CH_A2, ai_CH_A0, ai_CH_A1, di_SW0, di_SW1, di_SW2);
                g_run_para.user_paused = true;
            }
#endif//SIM_TARGETHW_RUN_TIME
#ifdef _SIM_REAL_TIME
            //update current hw time
            _UINT64 hw_time_diff = 0;
            if(hw_time < g_run_para.prev_hw_time)
                hw_time_diff = (0xffffffff-g_run_para.prev_hw_time)+hw_time;
            else
                hw_time_diff = hw_time - g_run_para.prev_hw_time;//in usecs
            if(((_UINT64)g_run_para.sim_stepsize*(_UINT64)SIM_TIME_RESOLUTION) <= (_UINT64)hw_time_diff*(_UINT64)1000ULL)
            {
                g_run_para.prev_hw_time = hw_time;
#else
            {
#endif//_SIM_REAL_TIME
                g_rootsig_obj.Run(&g_run_para, uart_shm, di_BTN0, ao_LED0_R, di_BTN1, ao_LED0_G, di_BTN2, ao_LED0_B, do_LED4, ao_LED1_B, ao_LED1_G, ao_LED1_R, ai_CH_A2, ai_CH_A0, ai_CH_A1, di_SW0, di_SW1, di_SW2);
                if((_UINT64)g_run_para.time_step_plot > (_UINT64)g_run_para.plot_stepsize)
                {
                    //g_rootsig_obj.RunPlotStep(&g_run_para, uart_shm, di_BTN0, ao_LED0_R, di_BTN1, ao_LED0_G, di_BTN2, ao_LED0_B, do_LED4, ao_LED1_B, ao_LED1_G, ao_LED1_R, ai_CH_A2, ai_CH_A0, ai_CH_A1, di_SW0, di_SW1, di_SW2);
                    g_run_para.time_step_plot = 0;
                }
                g_run_para.time_step_1 = g_run_para.time_step_0;
                g_run_para.time_step_0 = g_run_para.time_step_0 + g_run_para.sim_stepsize;
                g_run_para.time_step_plot = g_run_para.time_step_plot + g_run_para.plot_stepsize;
            }
#if (SIM_TARGETHW_RUN_DELAY > 0)
            delay(SIM_TARGETHW_RUN_DELAY);
#endif
#endif//UNDER_TESTING
	}
	g_system_initialized = 1;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef UNDER_TESTING
void setup(volatile _UINT32 *shm)
{
	comm_init(shm);
    for(int i=0; i<24; ++i)
        g_local_read_buf[i] = 0;
    g_write_size = 0;
    //
    a_o = 0;
    d_o = 0;
}

void loop(volatile _UINT32 *shm, AOUT_DTYPE &ao_LED1_B, AOUT_DTYPE &ao_LED1_G, AOUT_DTYPE &ao_LED1_R, AOUT_DTYPE &ao_LED0_B, AOUT_DTYPE &ao_LED0_G, AOUT_DTYPE &ao_LED0_R, _BIT &do_LED4)
{
    _UINT32 rsize = comm_available(shm);
    fpga_wait();
    if(rsize == 5)
    {
        comm_peek_array(g_local_read_buf, rsize, shm);
        comm_read_clear(shm);
        comm_write_clear(shm);
    	//
        a_o = g_local_read_buf[1];
        d_o = g_local_read_buf[2];
    	//
    	g_write_size = rsize;
    }
    fpga_wait();
    if(comm_write_idle(shm) && g_write_size > 0)
    {
        comm_write_array(g_local_read_buf, g_write_size, 0, shm);
        comm_write_flush(g_write_size, shm);
        g_write_size = 0;
    }
    fpga_wait();
    //
    ao0 = a_o;
    do0 = d_o;
}
#endif //UNDER_TESTING
//////////////////////////////////////////////////////////////////////////////////////////////////////////
_UINT64 millis()
{
    //_UINT64 ticks = *g_rt_clk_ticks;
    //fpga_wait();
    return (g_rt_clk_ticks >> 21);//20 - divide by 1048576
}

_UINT64 micros()
{
    //_UINT64 ticks = *g_rt_clk_ticks;
    //fpga_wait();
    return (g_rt_clk_ticks >> 12);//10 - divide by 1024
}

void delay(_UINT64 delay)
{
    _QWORD start_time = millis();
    while(1)
    {
        if((millis() - start_time) > delay)
            break;
    }
}

void delayMicroseconds(_UINT64 delay)
{
    _QWORD start_time = micros();
    while(1)
    {
        if((micros() - start_time) > delay)
            break;
    }
}

void fpga_wait()
{
#ifdef _BUILD_CMP_VIVADO_HLS
    //wait();
    //minimum 2. by trail and error. otherwise 2 bram ports are created for uart.
    //recommended is 3. uart will be smooth.
    ap_wait_n(DELAY_WAIT_CLKS);
#endif
}
///////////////////////////////////////////////////comm////////////////////////////////////////////////////////////////////////
#ifdef UNDER_TESTING
void comm_init(volatile _UINT32 *shm)
{
    for(_UINT32 i=0; i<COMM_BUF_SIZE; ++i) shm[i] = 0;
    fpga_wait();
}

_UINT32 comm_available(volatile _UINT32 *shm)
{
    _UINT32 val = shm[COMM_CTRL_RSIZE_IDX];
    fpga_wait();
    return val;
}

_INT32 comm_peek(_UINT32 idx, volatile _UINT32 *shm)
{
    _INT32 *val = (_INT32*)&shm[idx];
    fpga_wait();
    return *val;
}

void comm_peek_array(volatile _INT32 *buffer, _UINT32 size_to_read, volatile _UINT32 *shm)
{
    for(_UINT32 i=0; i<size_to_read; ++i)
    {
    	_INT32 *val = (_INT32*)&shm[i];
    	buffer[i] = *val;
    }
    fpga_wait();
}

void comm_read_clear(volatile _UINT32 *shm)
{
    shm[COMM_CTRL_RSIZE_IDX] = 0;
    fpga_wait();
}

bool comm_write_idle(volatile _UINT32 *shm)
{
    _UINT32 val = shm[COMM_CTRL_WSIZE_IDX];
    fpga_wait();
    return (val > 0) ? 0 : 1;
}

void comm_write(_INT32 data, _UINT32 offset_idx, volatile _UINT32 *shm)
{
	_INT32 *val = (_INT32*)&shm[offset_idx];
	*val = data;
    fpga_wait();
}

void comm_write_array(volatile _INT32 *data, _UINT32 size, _UINT32 offset_idx, volatile _UINT32 *shm)
{
    for(_UINT32 i=0; i<size; ++i)
    {
    	_INT32 *val = (_INT32*)&shm[offset_idx+i];
    	*val = data[i];
    }
    fpga_wait();
}

void comm_write_flush(_UINT32 wsize, volatile _UINT32 *shm)
{
    shm[COMM_CTRL_WSIZE_IDX] = wsize;
    fpga_wait();
}

void comm_write_clear(volatile _UINT32 *shm)
{
    shm[COMM_CTRL_WSIZE_IDX] = 0;
    fpga_wait();
}
#endif


///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/build/m1mta2ngk2ntawnaaxnjmzoqe2nzq2c5otcwnqcymtc2nwa4nte0k4odkyngyxndqx_build_tmp/FPGAComm169490209_542947950.cpp///////////////////////////////////////////////////////////////
/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "FPGAComm169490209_542947950.h"
#include "sim_helper.h"
#include "iodevice.h"
#include "fpga.h"

#ifdef FPGACOMM_PCIE
#define COMM_CTRL_SIZE          PCIE_CTRL_SIZE
#define COMM_DATA_SIZE          PCIE_SHM_SIZE-COMM_CTRL_SIZE
#define COMM_BUF_SIZE           PCIE_SHM_SIZE
#elif defined FPGACOMM_ETHERNET
#define COMM_CTRL_SIZE          ETH_CTRL_SIZE
#define COMM_DATA_SIZE          ETH_SHM_SIZE-COMM_CTRL_SIZE
#define COMM_BUF_SIZE           ETH_SHM_SIZE
#elif defined FPGACOMM_UART_542947950
#define COMM_CTRL_SIZE          UART_CTRL_SIZE
#define COMM_DATA_SIZE          UART_SHM_SIZE-COMM_CTRL_SIZE
#define COMM_BUF_SIZE           UART_SHM_SIZE
#elif defined FPGACOMM_SHM_0
#define COMM_CTRL_SIZE          SHM0_CTRL_SIZE
#define COMM_DATA_SIZE          SHM0_DATA_SIZE
#define COMM_BUF_SIZE           SHM0_DATA_SIZE+COMM_CTRL_SIZE
#elif defined FPGACOMM_SHM_1
#define COMM_CTRL_SIZE          SHM1_CTRL_SIZE
#define COMM_DATA_SIZE          SHM1_DATA_SIZE
#define COMM_BUF_SIZE           SHM1_DATA_SIZE+COMM_CTRL_SIZE
#elif defined FPGACOMM_SHM_2
#define COMM_CTRL_SIZE          SHM2_CTRL_SIZE
#define COMM_DATA_SIZE          SHM2_DATA_SIZE
#define COMM_BUF_SIZE           SHM2_DATA_SIZE+COMM_CTRL_SIZE
#elif defined FPGACOMM_SHM_3
#define COMM_CTRL_SIZE          SHM3_CTRL_SIZE
#define COMM_DATA_SIZE          SHM3_DATA_SIZE
#define COMM_BUF_SIZE           SHM3_DATA_SIZE+COMM_CTRL_SIZE
#else
#define COMM_CTRL_SIZE          2
#define COMM_DATA_SIZE          2046
#define COMM_BUF_SIZE           COMM_DATA_SIZE+COMM_CTRL_SIZE
#endif
//
#ifdef FPGACOMM_MASTER_542947950
#define COMM_CTRL_RSIZE_IDX     (COMM_BUF_SIZE-1)
#define COMM_CTRL_WSIZE_IDX     (COMM_BUF_SIZE-2)
#else
#define COMM_CTRL_RSIZE_IDX     (COMM_BUF_SIZE-2)
#define COMM_CTRL_WSIZE_IDX     (COMM_BUF_SIZE-1)
#endif

FPGAComm169490209_542947950::FPGAComm169490209_542947950(){
}

FPGAComm169490209_542947950::~FPGAComm169490209_542947950(){
}

void FPGAComm169490209_542947950::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
    m_prerun_count = 0;
}

void FPGAComm169490209_542947950::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, _INT32 *out, _INT32 *out_length, _INT32 *in, _INT32 *in_length, volatile COMM_SHM_DTYPE *shm){
    if(m_prerun_count++ != 1) return;
    comm_init(shm);
    //
    *out_length = -1;//-1-no send/no recv; 0-send/recv empty data; >0 send/recv with data present
    m_recvd_but_not_send = m_send_but_not_recvd = 0;
    m_data_received = 0;
    m_prev_send_recv_time = arg->rt_clk_tick;
    recv_timeout = recv_timeout * 1000;//convert to micros
}

void FPGAComm169490209_542947950::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _INT32 *out, _INT32 *out_length, _INT32 *in, _INT32 *in_length, volatile COMM_SHM_DTYPE *shm){
    _UINT64 mtick = arg->rt_clk_tick;
#if (SIM_TARGETHW_COMM_DELAY>0)
    if((mtick - m_prev_send_recv_time) < (SIM_TARGETHW_COMM_DELAY*1000))
        return;
#endif
    //data_exchange_type: 0-send only, 1-recv only, 2-send then recv, 3-recv then send
    //send data
    _INT32 in_len = *in_length;
    if(in_len > -1 && comm_write_idle(shm))
    {
        if((data_exchange_type == 0 && in_len > 0)
           || (data_exchange_type == 2 && m_send_but_not_recvd == false)
           || (data_exchange_type == 3 && m_recvd_but_not_send == true))
        {
            _BYTE cal_checksum = 0;
#ifdef FPGACOMM_ENABLE_CHECKSUM_542947950
            for(_INT32 i=0; i<in_len; ++i)
            {
                _UINT32 data = (_UINT32)in[i];
                cal_checksum += (data >> 0) & 0xff;
                cal_checksum += (data >> 8) & 0xff;
                cal_checksum += (data >> 16) & 0xff;
                cal_checksum += (data >> 24) & 0xff;
            }
#endif
            //header
            _UINT32 data = TARGETHW_COMM_START_BYTE;
            _UINT32 len_bytes = in_len << 2;//mul by 4
            data |= ((len_bytes<<8) & 0xffff00);
            data |= (((_UINT32)cal_checksum<<24) & 0xff000000);
            comm_write(data, 0, shm);
            //data
            comm_write_array(in, in_len, 1, shm);
            comm_write_flush(in_len+1, shm);
            //
            if(data_exchange_type == 3)
                m_recvd_but_not_send = false;
            else if(data_exchange_type == 2)
                m_send_but_not_recvd = true;
            m_prev_send_recv_time = mtick;
        }
    }
    //receive data
RECV_AGAIN:
    _INT32 out_len = *out_length;
    if(out_len > 0)out_len = 0;//-1 only after timeout
    _UINT32 available = comm_available(shm);
    if(available > 0)
    {
        if(data_exchange_type == 1
            || (data_exchange_type == 2 && m_send_but_not_recvd == true)
            || (data_exchange_type == 3 && m_recvd_but_not_send == false))
        {
            m_data_received = available;
            if(m_data_received == recv_data_size+1)//+1 for header
            {
                comm_peek_array(out, m_data_received, shm);
                comm_read_clear(shm);
                comm_write_clear(shm);
                //validate received data with checksum
                _UINT32 hdata = out[0];
                _INT32 len = (_INT32)((hdata >> 8) & 0x0000ffff);
                if((hdata & 0xff) == TARGETHW_COMM_START_BYTE)
                {
#ifdef FPGACOMM_ENABLE_CHECKSUM_542947950
                    _BYTE cal_checksum = 0;
                    for(_INT32 i=0; i<recv_data_size; ++i)
                    {
                        _UINT32 data = out[1+i];//+1 for header
                        cal_checksum += (data >> 0) & 0xff;
                        cal_checksum += (data >> 8) & 0xff;
                        cal_checksum += (data >> 16) & 0xff;
                        cal_checksum += (data >> 24) & 0xff;
                    }
                    if(cal_checksum == ((hdata >> 24) & 0xff))
#endif
                        out_len = (_INT32)recv_data_size;
                    m_prev_send_recv_time = mtick;
                    //
                    if(data_exchange_type == 3)
                        m_recvd_but_not_send = true;
                    else if(data_exchange_type == 2)
                        m_send_but_not_recvd = false;
                    m_data_received = 0;
                }
            }
        }
    }
    *out_length = out_len;
    //increment timeout
    if(out_len < 1)//)m_data_received > 0 || m_recvd_but_not_send == true || m_send_but_not_recvd == true)// || m_data_received > (recv_data_size+1))//+1 for header
    {
        _UINT32 pres = mtick;
        if((pres-m_prev_send_recv_time) > recv_timeout)//no need to check < condition
        {
            m_data_received = 0;
            m_recvd_but_not_send = false;
            m_send_but_not_recvd = false;
            //
            m_prev_send_recv_time = pres;
            //
            comm_read_clear(shm);
            comm_write_clear(shm);
        }
        //recv blocking
        if(recv_block/* && out_len < 1*/)
        {
            mtick = millis();
            goto RECV_AGAIN;
        }
    }
    else
    {
        m_prev_send_recv_time = mtick;
    }
}

void FPGAComm169490209_542947950::RunPlotStep(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _INT32 *out, _INT32 *out_length, _INT32 *in, _INT32 *in_length, volatile COMM_SHM_DTYPE *shm){
    //Run(arg, out, out_length, in, in_length, shm);
}

void FPGAComm169490209_542947950::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, _INT32 *out, _INT32 *out_length, _INT32 *in, _INT32 *in_length, volatile COMM_SHM_DTYPE *shm){
}

void FPGAComm169490209_542947950::comm_init(volatile _UINT32 *shm)
{
    for(_UINT32 i=0; i<COMM_BUF_SIZE; ++i) shm[i] = 0;
    fpga_wait();
}

_UINT32 FPGAComm169490209_542947950::comm_available(volatile _UINT32 *shm)
{
    _UINT32 val = shm[COMM_CTRL_RSIZE_IDX];
    fpga_wait();
    return val;
}

_INT32 FPGAComm169490209_542947950::comm_peek(_UINT32 idx, volatile _UINT32 *shm)
{
    _INT32 *val = (_INT32*)&shm[idx];
    fpga_wait();
    return *val;
}

void FPGAComm169490209_542947950::comm_peek_array(volatile _INT32 *buffer, _UINT32 size_to_read, volatile _UINT32 *shm)
{
    for(_UINT32 i=0; i<size_to_read; ++i)
    {
        _INT32 *val = (_INT32*)&shm[i];
        buffer[i] = *val;
    }
    fpga_wait();
}

void FPGAComm169490209_542947950::comm_read_clear(volatile _UINT32 *shm)
{
    shm[COMM_CTRL_RSIZE_IDX] = 0;
    fpga_wait();
}

bool FPGAComm169490209_542947950::comm_write_idle(volatile _UINT32 *shm)
{
    _UINT32 val = shm[COMM_CTRL_WSIZE_IDX];
    fpga_wait();
    return (val > 0) ? 0 : 1;
}

void FPGAComm169490209_542947950::comm_write(_INT32 data, _UINT32 offset_idx, volatile _UINT32 *shm)
{
    _INT32 *val = (_INT32*)&shm[offset_idx];
    *val = data;
    fpga_wait();
}

void FPGAComm169490209_542947950::comm_write_array(volatile _INT32 *data, _UINT32 size, _UINT32 offset_idx, volatile _UINT32 *shm)
{
    for(_UINT32 i=0; i<size; ++i)
    {
        _INT32 *val = (_INT32*)&shm[offset_idx+i];
        *val = data[i];
    }
    fpga_wait();
}

void FPGAComm169490209_542947950::comm_write_flush(_UINT32 wsize, volatile _UINT32 *shm)
{
    shm[COMM_CTRL_WSIZE_IDX] = wsize;
    fpga_wait();
}

void FPGAComm169490209_542947950::comm_write_clear(volatile _UINT32 *shm)
{
    shm[COMM_CTRL_WSIZE_IDX] = 0;
    fpga_wait();
}


///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/build/m1mta2ngk2ntawnaaxnjmzoqe2nzq2c5otcwnqcymtc2nwa4nte0k4odkyngyxndqx_build_tmp/PulseGenerator212657893_621712728.cpp///////////////////////////////////////////////////////////////
/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "PulseGenerator212657893_621712728.h"
#include "simcode.h"
#include "math.h"

PulseGenerator212657893_621712728::PulseGenerator212657893_621712728(){
}

PulseGenerator212657893_621712728::~PulseGenerator212657893_621712728(){
}

void PulseGenerator212657893_621712728::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
	m_prerun_count = 0;
}

void PulseGenerator212657893_621712728::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, OUT_DTYPE *out, _FXDPT_FLOAT *ctrl_duty){
    if(m_prerun_count++ != 1) return;
    //doc: frequency and duty cycle should be multiple of step_size
#ifdef PREFER_FIXED_POINT_MATH
    _INT64 freq = FREQUENCY_621712728;
    _INT64 duty = DUTY_CYCLE_621712728;
    _INT64 cycle_count = 0;
#if (RT_SRC_621712728 > 0)
    cycle_count = fxdpt_div(int_to_fxdpt((_INT64)1000), freq);//only millis is possible because 1000000 cannot be converted to fxdpt
    cycle_count = fxdpt_mul(int_to_fxdpt((_INT64)2), cycle_count);
#else
    cycle_count = fxdpt_div(int_to_fxdpt((_INT64)SIM_TIMESTEPS_PER_SEC), freq);
#endif
    m_on_count = fxdpt_to_int(fxdpt_mul(cycle_count, duty));
    m_cycle_count = fxdpt_to_int(cycle_count);
    m_count = 0;
    m_prev_micro = 0;
#endif
}

void PulseGenerator212657893_621712728::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, OUT_DTYPE *out, _FXDPT_FLOAT *ctrl_duty){
#ifdef PREFER_FIXED_POINT_MATH
#if (RT_SRC_621712728 > 0)
    _UINT64 milli = millis();//only millis is possible
    m_count += milli-m_prev_micro;
    m_prev_micro = milli;
#else
    ++m_count;
#endif
    OUT_DTYPE val = 0;
    if(m_count < m_on_count)
        val = OFFSET_621712728 + AMPLITUDE_621712728;
    else
    {
        val = OFFSET_621712728 - AMPLITUDE_621712728;
        if(m_count >= m_cycle_count)
            m_count = 0;
    }
    *out = val;
#else
    _FLOAT frac = FREQUENCY_621712728;
#if (RT_SRC_621712728 > 0)
    frac *= ((_FLOAT)micros() / (_FLOAT)1000000);
#else
    frac *= ((_FLOAT)arg->time_step_0 / (_FLOAT)SIM_TICKS_PER_SEC);
#endif
    frac = frac-floor(frac);
    //y=mx+c
#if (EX_CTRL_DUTY_CYCLE_621712728_621712728 > 0)
    if(frac < *ctrl_duty)
#else
    if(frac < DUTY_CYCLE_621712728)
#endif
        frac = 1;
    else
        frac = -1;
    *out = OFFSET_621712728 + (AMPLITUDE_621712728 * frac);
#endif
}

void PulseGenerator212657893_621712728::RunPlotStep(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, OUT_DTYPE *out, _FXDPT_FLOAT *ctrl_duty){
    Run(arg, out, ctrl_duty);
}

void PulseGenerator212657893_621712728::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, OUT_DTYPE *out, _FXDPT_FLOAT *ctrl_duty){
}


///////////////////////////////////////////////////////////C:/Users/AL/Desktop/XC7A35T_TEMPLATE_PROJ/CaspHlsDefault/build/bin/RootSignalClass.cpp///////////////////////////////////////////////////////////////
#include "matrix_math.h"
#include "RootSignalClass.h"
#include "sim_helper.h"
#include "simcode_global.h"
#include "simcode.h"

RootSignalClass::RootSignalClass(){
}

RootSignalClass::~RootSignalClass(){
}

void RootSignalClass::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
	prerun_count = 0;
	m_blk_id = arg->blk_id;
	m_exec_blks[0].const_count_down = 0;
	//
	BLOCK_CONSTRUCTOR_PARAM_STRUCT blkc;
	{
		blkc.nport_data = 0; blkc.port_data = 0;
		blkc.blk_id = 431281182;
		blkc.thread_idx = 0;
		obj431281182.Initialize(&blkc);
		for(int i=0;i<1;++i) obj431281182_in_down[i]=0;
	}
	{
		blkc.nport_data = 0; blkc.port_data = 0;
		blkc.blk_id = 635123591;
		blkc.thread_idx = 0;
		obj635123591.Initialize(&blkc);
		for(int i=0;i<1;++i) obj635123591_in_down[i]=0;
	}
	{
		blkc.nport_data = 0; blkc.port_data = 0;
		blkc.blk_id = 592814512;
		blkc.thread_idx = 0;
		obj592814512.Initialize(&blkc);
		for(int i=0;i<1;++i) obj592814512_in_down[i]=0;
	}
	{
		blkc.nport_data = 0; blkc.port_data = 0;
		blkc.blk_id = 621712728;
		blkc.thread_idx = 0;
		obj621712728.Initialize(&blkc);
		for(int i=0;i<1;++i) obj621712728_ctrl_ratio[i]=0;
	}
	{
		blkc.nport_data = 0; blkc.port_data = 0;
		blkc.blk_id = 542947950;
		blkc.thread_idx = 0;
		obj542947950.Initialize(&blkc);
	}
}

void RootSignalClass::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, volatile _UINT32 *uart, _BIT pin_BTN0, _UINT32 &pin_LED0_R, _BIT pin_BTN1, _UINT32 &pin_LED0_G, _BIT pin_BTN2, _UINT32 &pin_LED0_B, _BIT &pin_LED4, _UINT32 &pin_LED1_B, _UINT32 &pin_LED1_G, _UINT32 &pin_LED1_R, _UINT32 pin_CH_A2, _UINT32 pin_CH_A0, _UINT32 pin_CH_A1, _BIT pin_SW0, _BIT pin_SW1, _BIT pin_SW2){
	++prerun_count;
	//
	//
	obj431281182.input_low_threshold = 0L;
	obj431281182.input_high_threshold = 1L;
	obj431281182.init_value = 0L;
	obj431281182.inc_dec_val = 50L;
	obj431281182.count_on = 1;
	obj431281182.lower_limit = 0L;
	obj431281182.upper_limit = 1000L;
	obj431281182.rst_on_limits = 1;
	if(prerun_count>1)obj431281182.PreRun(arg, obj511934590_out_port, obj431281182_in_down, obj431281182_out);
	//
	//
	//
	obj635123591.input_low_threshold = 0L;
	obj635123591.input_high_threshold = 1L;
	obj635123591.init_value = 0L;
	obj635123591.inc_dec_val = 50L;
	obj635123591.count_on = 1;
	obj635123591.lower_limit = 0L;
	obj635123591.upper_limit = 1000L;
	obj635123591.rst_on_limits = 1;
	if(prerun_count>1)obj635123591.PreRun(arg, obj69315064_out_port, obj635123591_in_down, obj635123591_out);
	//
	//
	//
	obj592814512.input_low_threshold = 0L;
	obj592814512.input_high_threshold = 1L;
	obj592814512.init_value = 0L;
	obj592814512.inc_dec_val = 50L;
	obj592814512.count_on = 1;
	obj592814512.lower_limit = 0L;
	obj592814512.upper_limit = 1000L;
	obj592814512.rst_on_limits = 1;
	if(prerun_count>1)obj592814512.PreRun(arg, obj478833356_out_port, obj592814512_in_down, obj592814512_out);
	//
	//
	if(prerun_count>1)obj621712728.PreRun(arg, obj621712728_out, obj621712728_ctrl_ratio);
	//
	//
	obj160609486_o[0] = 4;
	//
	for(int i=0; i<4; ++i) obj861899881_o[i] = 0;
	//
	obj542947950.data_exchange_type = 3;
	obj542947950.send_data_size = 4L;
	obj542947950.recv_data_size = 4L;
	obj542947950.recv_timeout = 1000L;
	obj542947950.recv_block = 0;
	if(prerun_count>1)obj542947950.PreRun(arg, obj542947950_out, obj542947950_out_length, obj861899881_o, obj160609486_o, uart);
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	obj26391711_out[0] = 0;
	//
	//
	GroupCustomBlkData();
}

void RootSignalClass::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, volatile _UINT32 *uart, _BIT pin_BTN0, _UINT32 &pin_LED0_R, _BIT pin_BTN1, _UINT32 &pin_LED0_G, _BIT pin_BTN2, _UINT32 &pin_LED0_B, _BIT &pin_LED4, _UINT32 &pin_LED1_B, _UINT32 &pin_LED1_G, _UINT32 &pin_LED1_R, _UINT32 pin_CH_A2, _UINT32 pin_CH_A0, _UINT32 pin_CH_A1, _BIT pin_SW0, _BIT pin_SW1, _BIT pin_SW2){
	Run0(arg, uart, pin_BTN0, pin_LED0_R, pin_BTN1, pin_LED0_G, pin_BTN2, pin_LED0_B, pin_LED4, pin_LED1_B, pin_LED1_G, pin_LED1_R, pin_CH_A2, pin_CH_A0, pin_CH_A1, pin_SW0, pin_SW1, pin_SW2);
}

void RootSignalClass::Run0(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, volatile _UINT32 *uart, _BIT pin_BTN0, _UINT32 &pin_LED0_R, _BIT pin_BTN1, _UINT32 &pin_LED0_G, _BIT pin_BTN2, _UINT32 &pin_LED0_B, _BIT &pin_LED4, _UINT32 &pin_LED1_B, _UINT32 &pin_LED1_G, _UINT32 &pin_LED1_R, _UINT32 pin_CH_A2, _UINT32 pin_CH_A0, _UINT32 pin_CH_A1, _BIT pin_SW0, _BIT pin_SW1, _BIT pin_SW2){
	obj511934590_out_port[0] = (pin_BTN0>0)?1:0;
	CopyArray(obj26391711_out, (_INT32*)obj861899881_o, 1);
	CopyArray(obj910839818_out_port, (_INT32*)(&obj861899881_o[1]), 1);
	CopyArray(obj942422658_out_port, (_INT32*)(&obj861899881_o[2]), 1);
	obj431281182.Run(arg, obj511934590_out_port, obj431281182_in_down, obj431281182_out);
	_INT32 v841180990_in_port = (obj431281182_out[0] * 1) + 0;
	pin_LED0_R = (_UINT32)(v841180990_in_port);
	obj69315064_out_port[0] = (pin_BTN1>0)?1:0;
	obj635123591.Run(arg, obj69315064_out_port, obj635123591_in_down, obj635123591_out);
	_INT32 v216394639_in_port = (obj635123591_out[0] * 1) + 0;
	pin_LED0_G = (_UINT32)(v216394639_in_port);
	obj478833356_out_port[0] = (pin_BTN2>0)?1:0;
	obj592814512.Run(arg, obj478833356_out_port, obj592814512_in_down, obj592814512_out);
	_INT32 v718312223_in_port = (obj592814512_out[0] * 1) + 0;
	pin_LED0_B = (_UINT32)(v718312223_in_port);
	obj621712728.Run(arg, obj621712728_out, obj621712728_ctrl_ratio);
	if(obj621712728_out[0]<=0)pin_LED4 = (_BIT)( 0); else if(obj621712728_out[0]>=1)pin_LED4 = (_BIT)( 1);
	obj542947950.Run(arg, obj542947950_out, obj542947950_out_length, obj861899881_o, obj160609486_o, uart);
	CopyArray((&obj542947950_out[1]), obj996299800_tap_out1, 1);
	CopyArray(&(&obj542947950_out[1])[1], obj996299800_tap_out2, 1);
	CopyArray(&(&obj542947950_out[1])[2], obj996299800_tap_out3, 1);
	_INT32 v3120380_in_port = (obj996299800_tap_out3[0] * 1) + 0;
	pin_LED1_B = (_UINT32)(v3120380_in_port);
	_INT32 v211972937_in_port = (obj996299800_tap_out2[0] * 1) + 0;
	pin_LED1_G = (_UINT32)(v211972937_in_port);
	_INT32 v1017907469_in_port = (obj996299800_tap_out1[0] * 1) + 0;
	pin_LED1_R = (_UINT32)(v1017907469_in_port);
	obj953124361_out_port[0] = pin_CH_A2;
	obj953124361_out_port[0] = (obj953124361_out_port[0] * 1) + 0;
	CopyArray(obj953124361_out_port, (_INT32*)(&obj861899881_o[3]), 1);
	obj910839818_out_port[0] = pin_CH_A0;
	obj910839818_out_port[0] = (obj910839818_out_port[0] * 1) + 0;
	obj942422658_out_port[0] = pin_CH_A1;
	obj942422658_out_port[0] = (obj942422658_out_port[0] * 1) + 0;
	obj305742853_out_port[0] = (pin_SW0>0)?1:0;
	obj694304554_out_port[0] = (pin_SW1>0)?1:0;
	obj793849453_out_port[0] = (pin_SW2>0)?1:0;
	obj26391711_out[0] = 0;
	obj26391711_out[0] = INSERT_VAL(obj305742853_out_port[0], obj26391711_out[0], 1, 0);
	obj26391711_out[0] = INSERT_VAL(obj694304554_out_port[0], obj26391711_out[0], 1, 1);
	obj26391711_out[0] = INSERT_VAL(obj793849453_out_port[0], obj26391711_out[0], 1, 2);
	//
}

void RootSignalClass::RunPlotStep(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, volatile _UINT32 *uart, _BIT pin_BTN0, _UINT32 &pin_LED0_R, _BIT pin_BTN1, _UINT32 &pin_LED0_G, _BIT pin_BTN2, _UINT32 &pin_LED0_B, _BIT &pin_LED4, _UINT32 &pin_LED1_B, _UINT32 &pin_LED1_G, _UINT32 &pin_LED1_R, _UINT32 pin_CH_A2, _UINT32 pin_CH_A0, _UINT32 pin_CH_A1, _BIT pin_SW0, _BIT pin_SW1, _BIT pin_SW2){
}

void RootSignalClass::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, volatile _UINT32 *uart, _BIT pin_BTN0, _UINT32 &pin_LED0_R, _BIT pin_BTN1, _UINT32 &pin_LED0_G, _BIT pin_BTN2, _UINT32 &pin_LED0_B, _BIT &pin_LED4, _UINT32 &pin_LED1_B, _UINT32 &pin_LED1_G, _UINT32 &pin_LED1_R, _UINT32 pin_CH_A2, _UINT32 pin_CH_A0, _UINT32 pin_CH_A1, _BIT pin_SW0, _BIT pin_SW1, _BIT pin_SW2){
	obj542947950.PostRun(arg, obj542947950_out, obj542947950_out_length, obj861899881_o, obj160609486_o, uart);
	obj621712728.PostRun(arg, obj621712728_out, obj621712728_ctrl_ratio);
	obj592814512.PostRun(arg, obj478833356_out_port, obj592814512_in_down, obj592814512_out);
	obj635123591.PostRun(arg, obj69315064_out_port, obj635123591_in_down, obj635123591_out);
	obj431281182.PostRun(arg, obj511934590_out_port, obj431281182_in_down, obj431281182_out);
}

void* RootSignalClass::GetInstCtrlDispDataPointer(_UINT32 id, int pt_idx){
	return 0;
}

void RootSignalClass::SetPlotWindowPointer(_UINT32 id, int data_index, char *plot_wnd, int wnd_type){
}

void RootSignalClass::GroupCustomBlkData(){
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
	//
}


///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/src_fg/fxd_pt_math.cpp///////////////////////////////////////////////////////////////
/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "fxd_pt_math.h"

_INT fxdpt_sqrt(_INT p_Square)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT  res;
    _INT  delta;
    _INT  maxError;

	if ( p_Square <= 0 )
		return 0;

	/* start at half */
	res = (p_Square >> 1);

	/* determine allowable error */
    maxError =  fxdpt_mul(p_Square, FXDPT_SQRT_ERR);
	do
	{
        delta =  (fxdpt_mul( res, res ) - p_Square);
        res -=  fxdpt_div(delta, ( res << 1 ));
	}
	while ( delta > maxError || delta < -maxError );

	return res;
}

_INT fxdpt_log(_INT p_Base)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT w = 0;
    _INT y = 0;
    _INT z = 0;
    _INT num = _ONE;
    _INT dec = 0;

    if ( p_Base == _ONE)
		return 0;

	if ( p_Base == 0 )
		return 0xffffffff;

    for ( dec=0 ; fxdpt_abs( p_Base ) >= _TWO ; dec += _ONE )
        p_Base = fxdpt_div(p_Base, FXDPT_LNE);

    p_Base -= _ONE;
	z = p_Base;
	y = p_Base;
    w = _ONE;

	while ( y != y + w )
	{
        z = 0 - fxdpt_mul( z , p_Base );
        num += _ONE;
        w = fxdpt_div( z , num );
		y += w;
	}

	return y + dec;
}

_INT fxdpt_log10(_INT p_Base)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    // ln(p_Base)/ln(10)
    // more accurately, ln(p_Base) * (1/ln(10))
    _INT log = fxdpt_log(p_Base);
    return fxdpt_mul(log, FXDPT_1OLN10);
}

_INT fxdpt_exp(_INT p_Base)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT w;
    _INT y;
    _INT num;

    for ( w=_ONE, y=_ONE, num=_ONE ; y != y+w ; num += _ONE )
	{
        w = fxdpt_mul(w, fxdpt_div(p_Base, num));
		y += w;
	}

	return y;
}

_INT fxdpt_pow(_INT p_Base, _INT p_Power)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    if ( p_Base < 0 && fxdpt_mod(p_Power, _TWO) != (_FXDPT_FLOAT)0 )
        return - fxdpt_exp( fxdpt_mul(p_Power, fxdpt_log( -p_Base )) );
	else
        return fxdpt_exp( fxdpt_mul(p_Power, fxdpt_log(fxdpt_abs( p_Base ))) );
}

static _INT sinx(_INT x)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT xpwr;
    _INT64 xftl;
    _INT xresult;
	short positive;

	xresult = 0;
	xpwr = x;
    xftl = 1;
    positive = -1;

	// Note: 12! largest for long
    for (int i = 1; i < 7; i+=2)
	{
        _INT r1 = fxdpt_div(xpwr, int_to_fxdpt(xftl));
        if(positive)
            xresult += r1;
        else
            xresult -= r1;
        xpwr = fxdpt_mul(x, xpwr);
        xpwr = fxdpt_mul(x, xpwr);
		xftl *= i+1;
		xftl *= i+2;
		positive = !positive;
	}

	return xresult;
}

_INT fxdpt_sin(_INT theta)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT xresult;
	short bBottom = 0;
	//static fixed xPI = XPI;
	//static fixed x2PI = X2PI;
	//static fixed xPIO2 = XPIO2;

    _INT x = fxdpt_mod(theta, (_INT)FXDPT_2PI);//dont remove type conversion
	if ( x < 0 )
        x += FXDPT_2PI;

    if ( x > FXDPT_PI )
	{
		bBottom = -1;
        x -= FXDPT_PI;
	}

    if ( x <= FXDPT_PIO2 )
		xresult = sinx(x);
	else
        xresult = sinx(FXDPT_PIO2-(x-FXDPT_PIO2));

	if ( bBottom )
		return -xresult;

	return xresult;
}

_INT fxdpt_cos(_INT theta)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    return fxdpt_sin(theta + FXDPT_PIO2);
}

_INT fxdpt_tan(_INT theta)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT sin = fxdpt_sin(theta);
    _INT cos = fxdpt_cos(theta);
    return fxdpt_div(sin, cos);
}

#define FXDPT_ATAN2_ITERS       12
static const _INT FXDPT_ATAN[FXDPT_ATAN2_ITERS] = {
    real_to_fxdpt(45.000),		// atan(1/1)
    real_to_fxdpt(26.565),		// atan(1/2)
    real_to_fxdpt(14.036),		// atan(1/4)
    real_to_fxdpt(7.125),		// atan(1/8)
    real_to_fxdpt(3.576),		// atan(1/16)
    real_to_fxdpt(1.790),		// atan(1/32)
    real_to_fxdpt(0.895),		// atan(1/64)
    real_to_fxdpt(0.448),		// atan(1/128)
    real_to_fxdpt(0.224),		// atan(1/256)
    real_to_fxdpt(0.112),		// atan(1/512)
    real_to_fxdpt(0.056),		// atan(1/1024)
    real_to_fxdpt(0.028),		// atan(1/2048)
};
_INT fxdpt_atan2(_INT y, _INT x)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT result = 0;
    _UINT ux, uy, dy, dr;
    dr = 0;
    if(x == 0 && y == 0)
        return 0;
    if(y < 0)
    {
        x = -x;
        uy = -y;
        result = -_ONE80;
    }
    else
        uy = y;
    if(x < 0)
    {
        ux = uy;
        uy = -x;
        result += -_NINTY;
    }
    else
        ux = x;
    while(ux < FXDPT_RESOLUTION && uy < FXDPT_RESOLUTION)
    {
        ux <<= 1;
        uy <<= 1;
    }
    for(_UINT i=0; i<FXDPT_ATAN2_ITERS; ++i)
    {
        dy = ux >> i;
        if(uy > dy)
        {
            ux += uy >> i;
            uy -= dy;
            dr = FXDPT_ATAN[i];
            result += dr;
        }
    }
    return result + (dr >> 1);
}

int fxdpt_str(_INT n, char *buf)
{
#ifdef _BUILD_CMP_VIVADO_HLS
#pragma HLS ALLOCATION instances=mul limit=1 operation
#pragma HLS ALLOCATION instances=sdiv limit=1 operation
#endif
    _INT64 ipart = fxdpt_int_part(n);
    _INT64 fpart = fxdpt_frac_part(n);
    _INT64 intdigits = 0;

	int i = 0;
	int j = 0;
	int d = 0;

	int offset = 0;

    _INT64 v;
    _INT64 t;
    _INT64 st = 0;

	if (n != 0)
	{
        intdigits = fxdpt_int_part(fxdpt_ceil(fxdpt_log10(fxdpt_abs(n))));
	}

	if (intdigits < 1) intdigits = 1;

	offset = intdigits - 1;

	if (n < 0)
	{
		buf[0] = '-';
		offset++;
		n = -n;
		ipart = -ipart;
		fpart = -fpart;
	}

	// integer portion

	for (i = 0; i < intdigits; i++)
	{
		j = offset - i;
		d = ipart % 10;
		ipart = ipart / 10;
		buf[j] = '0' + d;
	}

	// decimal point

	buf[offset + 1] = '.';

	// fractional portion

	v = 5;
    t = FXDPT_RESOLUTION >> 1;

    for (i = 0; i < FXDPT_STR_DEC_DIGITS - 1; i++)
	{
		v = (v << 1) + (v << 3);
	}

    for (i = 0; i < FXDPT_FRAC_BITS; i++)
	{
		if (fpart & t)
		{
			st += v;
		}
		v = v >> 1;
		t = t >> 1;
	}

    offset += FXDPT_STR_DEC_DIGITS + 1;

    for (i = 0; i < FXDPT_STR_DEC_DIGITS; i++)
	{
		j = offset - i;
		d = st % 10;
		st = st / 10;
		buf[j] = '0' + d;
	}

	// ending zero
	buf[offset + 1] = '\0';

    return offset + 2;
}


///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/src_fg/sim_helper_fg.cpp///////////////////////////////////////////////////////////////
/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include <stdlib.h>
#include "sim_helper_fg.h"
#include "sim_target.h"
#include "simcode.h"
#include "simcode_global.h"

_BYTE SplitStringToBytes(const char *str, const char seperator_character, _BYTE *out, _BYTE out_size)
{
    char num[4];
    int str_idx = 0;
    int num_idx = 0;
    int count = 0;
    while(1)
    {
        if(str[str_idx] == seperator_character || str[str_idx] == 0)
        {
            num[num_idx] = 0;
            if(out)
            {
                if(count < out_size)
                    out[count] = (_BYTE)atoi(num);
            }
            ++count;
            num_idx = 0;
            if(str[str_idx] == 0)
                break;
        }
        else
        {
            num[num_idx] = str[str_idx];
            ++num_idx;
        }
        ++str_idx;
    }
    return count;
}

int SplitStringToInts(const char *str, const char seperator_character, _INT16 *out, const int out_size)
{
    char num[12];
    int str_idx = 0;
    int num_idx = 0;
    int count = 0;
    while(1)
    {
        if(str[str_idx] == seperator_character || str[str_idx] == 0)
        {
            num[num_idx] = 0;
            if(out)
            {
                if(count < out_size)
                    out[count] = atoi(num);//(int)strtol(num, NULL, 10);
            }
            ++count;
            num_idx = 0;
            if(str[str_idx] == 0)
                break;
        }
        else
        {
            num[num_idx] = str[str_idx];
            ++num_idx;
        }
        ++str_idx;
    }
    return count;
}

int SplitStringToInts(const char *str, const char seperator_character, _INT32 *out, const int out_size)
{
    char num[12];
    int str_idx = 0;
    int num_idx = 0;
    int count = 0;
    while(1)
    {
        if(str[str_idx] == seperator_character || str[str_idx] == 0)
        {
            num[num_idx] = 0;
            if(out)
            {
                if(count < out_size)
                    out[count] = atoi(num);//(int)strtol(num, NULL, 10);
            }
            ++count;
            num_idx = 0;
            if(str[str_idx] == 0)
                break;
        }
        else
        {
            num[num_idx] = str[str_idx];
            ++num_idx;
        }
        ++str_idx;
    }
    return count;
}

int SplitStringToInts(const char *str, const char seperator_character, _INT64 *out, const int out_size)
{
    char num[12];
    int str_idx = 0;
    int num_idx = 0;
    int count = 0;
    while(1)
    {
        if(str[str_idx] == seperator_character || str[str_idx] == 0)
        {
            num[num_idx] = 0;
            if(out)
            {
                if(count < out_size)
                    out[count] = atol(num);//strtol(num, NULL, 10);
            }
            ++count;
            num_idx = 0;
            if(str[str_idx] == 0)
                break;
        }
        else
        {
            num[num_idx] = str[str_idx];
            ++num_idx;
        }
        ++str_idx;
    }
    return count;
}

int SplitStringToFloats(const char *str, const char seperator_character, _FXDPT_FLOAT32 *out, const int out_size)
{
    char num[18];
    int str_idx = 0;
    int num_idx = 0;
    int count = 0;
    while(1)
    {
        if(str[str_idx] == seperator_character || str[str_idx] == 0)
        {
            num[num_idx] = 0;
            if(out)
            {
                if(count < out_size)
                {
#ifdef PREFER_FIXED_POINT_MATH
                    out[count] = atoi(num);//strtof(num, NULL);
#else
                    out[count] = (float)atof(num);//strtof(num, NULL);
#endif
                    ++count;
                }
            }
            num_idx = 0;
            if(str[str_idx] == 0)
                break;
        }
        else
        {
            num[num_idx] = str[str_idx];
            ++num_idx;
        }
        ++str_idx;
    }
    return count;
}

void CopyByteArray(const _BYTE *in, _BYTE *out, int in_size, _BOOLEAN reverse_order, _BOOLEAN align_bits, _BOOLEAN align_expand)
{
#if (MEM_BYTE_SIZE == 16)
    _UINT32 idx = 0, count = 0;
    _BYTE byte = 0, even_num = 0;
    if(align_bits == 1 && align_expand == 1)
        in_size /= 2;
    if(in_size%2 == 0)
        even_num = 1;
    for(int i=0; i<in_size; ++i)
    {
        if(reverse_order)
            byte = in[(in_size-1)-i] & 0xff;
        else
            byte = in[i] & 0xff;
        if(align_bits == 1 && align_expand == 0)
        {
            _BYTE &obyte = out[idx];
            if(count % 2)
            {
                obyte |= ((byte << 8) & 0xff00);
                ++idx;
            }
            else
                obyte = byte & 0xff;
            ++count;
        }
        else if(align_bits == 1 && align_expand == 1)//out size shall be double that of in size
        {
            out[idx++] = byte & 0xff;
            if(even_num)
                out[idx++] = (byte >> 8) & 0xff;
        }
        else
           out[idx++] = byte;
    }
#else
    for(int i=0; i<in_size; ++i)
    {
        if(reverse_order)
            out[i] = in[(in_size-1)-i];
        else
            out[i] = in[i];
    }
#endif
}

void CopyString(char *out, const char *in, int size)
{
    int idx = 0;
    while(1)
    {
        if(idx == size)
            break;
        out[idx] = in[idx];
        if(in[idx] == 0)
            break;
        ++idx;
    }
}

_BOOLEAN IsZcdDetected(_FXDPT_FLOAT64 pres, _FXDPT_FLOAT64 prev, _FXDPT_FLOAT64 ref, int direction)
{
    if(direction < 0)
    {
        if(pres < ref && prev > ref)
            return 1;
    }
    else
    {
        if(pres > ref && prev < ref)
            return 1;
    }
    return 0;
}

_BOOLEAN IsZcdDetected(_FXDPT_FLOAT32 pres, _FXDPT_FLOAT32 prev, _FXDPT_FLOAT32 ref, int direction)
{
    if(direction < 0)
    {
        if(pres < ref && prev > ref)
            return 1;
    }
    else
    {
        if(pres > ref && prev < ref)
            return 1;
    }
    return 0;
}

_FXDPT_FLOAT GetMinimumDistance(const XYZP *ref, const XYZP *p1, const XYZP *p2)
{
    _FXDPT_FLOAT d1 = GetDistance(ref, p1);
    _FXDPT_FLOAT d2 = GetDistance(ref, p2);
    return (d1 < d2)?d1:d2;
}

_FXDPT_FLOAT GetMaximumDistance(const XYZP *ref, const XYZP *p1, const XYZP *p2)
{
    _FXDPT_FLOAT d1 = GetDistance(ref, p1);
    _FXDPT_FLOAT d2 = GetDistance(ref, p2);
    return (d1 > d2)?d1:d2;
}

void GetPointOnLine(const XYZP *ref, const XYZP *old_pt, _FXDPT_FLOAT new_disref, XYZP *new_pt)
{
#ifdef PREFER_FIXED_POINT_MATH
    _FXDPT_FLOAT ratio = fxdpt_div(new_disref, GetDistance(ref, old_pt));
    new_pt->x = ref->x + fxdpt_mul((old_pt->x - ref->x), ratio);
    new_pt->y = ref->y + fxdpt_mul((old_pt->y - ref->y), ratio);
    new_pt->z = ref->z + fxdpt_mul((old_pt->z - ref->z), ratio);
#else
    _FXDPT_FLOAT ratio = new_disref/GetDistance(ref, old_pt);
    new_pt->x = ref->x + (old_pt->x-ref->x)*ratio;
    new_pt->y = ref->y + (old_pt->y-ref->y)*ratio;
    new_pt->z = ref->z + (old_pt->z-ref->z)*ratio;
#endif
}

void AngleBetweenTwoPoints(const XYZP *org, const XYZP *p1, const XYZP *p2, XYZP *ang)
{
    ang->x = AngleFromXAxis(org->y, org->z, p2->y, p2->z);
    ang->y = AngleFromXAxis(org->z, org->x, p2->z, p2->x);
    ang->z = AngleFromXAxis(org->x, org->y, p2->x, p2->y);
    //
    ang->x -= AngleFromXAxis(org->y, org->z, p1->y, p1->z);
    ang->y -= AngleFromXAxis(org->z, org->x, p1->z, p1->x);
    ang->z -= AngleFromXAxis(org->x, org->y, p1->x, p1->y);
}

_FXDPT_FLOAT AngleFromXAxis(_FXDPT_FLOAT x1, _FXDPT_FLOAT y1, _FXDPT_FLOAT x2, _FXDPT_FLOAT y2)
{
    //calculate angle wrt x-axis taking x1,y1 as origin
    //range 0 to 360 degres, no negative angles
#ifdef PREFER_FIXED_POINT_MATH
    _FXDPT_FLOAT y = y2 - y1;
    _FXDPT_FLOAT x = x2 - x1;
    _FXDPT_FLOAT m = 0;
    _BYTE x_zero = IsZero(x);
    _BYTE y_zero = IsZero(y);
    if(x_zero && y_zero)
        return 0;
    if(x_zero)
    {
        if(y2 > y1)
            m = int_to_fxdpt(90);
        else
            m = int_to_fxdpt(-90);
    }
    else if(y_zero)
    {
        if(x2 > x1)
            m = 0;
        else
            m = int_to_fxdpt(180);
    }
    else
    {
        m = fxdpt_atan2(y,x);
        if(m < int_to_fxdpt(-90) && m > int_to_fxdpt(-180))
            m = m + int_to_fxdpt(360);
    }
#else
    _FLOAT y = y2-y1;
    _FLOAT x = x2-x1;
    _FLOAT m = 0;
    _BYTE x_zero = IsZero(x);
    _BYTE y_zero = IsZero(y);
    if(x_zero && y_zero)
        return 0;
    if(x_zero)
    {
        if(y2 > y1)
            m = 90;
        else
            m = -90;
    }
    else if(y_zero)
    {
        if(x2 > x1)
            m = 0;
        else
            m = 180;
    }
    else
    {
        m = atan2(y,x)*MATH_RAD_TO_DEG;
        if(m < -90 && m > -180)
            m = m + 360;
    }
#endif
    return m;
}

void Helper_ApplyOffset(XYZP *pt, const XYZP &off)
{
    /*if(*x >= DINFINITY || *x <= -DINFINITY
        || *y >= DINFINITY || *y <= -DINFINITY
        || *z >= DINFINITY || *z <= -DINFINITY)
        return;*/
    pt->x = pt->x + off.x;
    pt->y = pt->y + off.y;
    pt->z = pt->z + off.z;
}

void Helper_ApplyRotation(XYZP *pt, const XYZP *ang, const XYZP *org)
{
    /*if(*x >= DINFINITY || *x <= -DINFINITY
        || *y >= DINFINITY || *y <= -DINFINITY
        || *z >= DINFINITY || *z <= -DINFINITY)
        return;*/
#ifdef PREFER_FIXED_POINT_MATH
    //first translate to angle origin
    pt->x = pt->x - org->x;
    pt->y = pt->y - org->y;
    pt->z = pt->z - org->z;
    //apply rotation
    _FXDPT_FLOAT tx,ty,tz;
    tx = pt->x; ty = pt->y; tz = pt->z;
    //along x-axis
    _FXDPT_FLOAT temp = MATH_DEG_TO_RAD;
    if(!IsZero(ang->x))
    {
        temp = fxdpt_mul(ang->x, temp);
        _FXDPT_FLOAT cos_th = fxdpt_cos(temp);
        _FXDPT_FLOAT sin_th = fxdpt_sin(temp);
        pt->y = fxdpt_mul(ty,cos_th) - fxdpt_mul(tz,sin_th);
        pt->z = fxdpt_mul(ty,sin_th) + fxdpt_mul(tz,cos_th);
    }
    //along y-axis
    if(!IsZero(ang->y))
    {
        temp = fxdpt_mul(ang->y, temp);
        _FXDPT_FLOAT cos_th = fxdpt_cos(temp);
        _FXDPT_FLOAT sin_th = fxdpt_sin(temp);
        pt->x = fxdpt_mul(tx,cos_th) + fxdpt_mul(tz,sin_th);
        pt->z = -fxdpt_mul(tx,sin_th) + fxdpt_mul(tz,cos_th);
    }
    //along z-axis
    if(!IsZero(ang->z))
    {
        temp = fxdpt_mul(ang->z, temp);
        _FXDPT_FLOAT cos_th = fxdpt_cos(temp);
        _FXDPT_FLOAT sin_th = fxdpt_sin(temp);
        pt->x = fxdpt_mul(tx,cos_th) - fxdpt_mul(ty,sin_th);
        pt->y = fxdpt_mul(tx,sin_th) + fxdpt_mul(ty,cos_th);
    }
    //tranlate back to original co-ordinates
    pt->x = pt->x + org->x;
    pt->y = pt->y + org->y;
    pt->z = pt->z + org->z;
#else
    //first translate to angle origin
    pt->x -= org->x;
    pt->y -= org->y;
    pt->z -= org->z;
    //apply rotation
    _FLOAT tx,ty,tz;
    tx = pt->x; ty = pt->y; tz = pt->z;
    //along x-axis
    if(!IsZero(ang->x))
    {
        //tx = pt->x; ty = pt->y; tz = pt->z;
        _FLOAT cos_th = cos(ang->x*(_FLOAT)MATH_DEG_TO_RAD);
        _FLOAT sin_th = sin(ang->x*(_FLOAT)MATH_DEG_TO_RAD);
        pt->y = ty*cos_th - tz*sin_th;
        pt->z = ty*sin_th + tz*cos_th;
    }
    //along y-axis
    if(!IsZero(ang->y))
    {
        //tx = pt->x; ty = pt->y; tz = pt->z;
        _FLOAT cos_th = cos(ang->y*(_FLOAT)MATH_DEG_TO_RAD);
        _FLOAT sin_th = sin(ang->y*(_FLOAT)MATH_DEG_TO_RAD);
        pt->x = tx*cos_th + tz*sin_th;
        pt->z = -tx*sin_th + tz*cos_th;
    }
    //along z-axis
    if(!IsZero(ang->z))
    {
        //tx = pt->x; ty = pt->y; tz = pt->z;
        _FLOAT cos_th = cos(ang->z*(_FLOAT)MATH_DEG_TO_RAD);
        _FLOAT sin_th = sin(ang->z*(_FLOAT)MATH_DEG_TO_RAD);
        pt->x = tx*cos_th - ty*sin_th;
        pt->y = tx*sin_th + ty*cos_th;
    }
    //tranlate back to original co-ordinates
    pt->x += org->x;
    pt->y += org->y;
    pt->z += org->z;
#endif
}

void Helper_ApplyScale(XYZP *pt, const XYZP &sc)
{
    /*if(*x >= DINFINITY || *x <= -DINFINITY
        || *y >= DINFINITY || *y <= -DINFINITY
        || *z >= DINFINITY || *z <= -DINFINITY)
        return;*/
#ifdef PREFER_FIXED_POINT_MATH
    pt->x = fxdpt_mul(pt->x, sc.x);
    pt->y = fxdpt_mul(pt->y, sc.y);
    pt->z = fxdpt_mul(pt->z, sc.z);
#else
    pt->x *= sc.x;
    pt->y *= sc.y;
    pt->z *= sc.z;
#endif
}
////////////////////////////////////////////////////////////debounce////////////////////////////////////////////////////////
void SimDebounce::Initialize(_UINT64 timeout, _BIT init_output)
{
    m_timeout = timeout;
    m_init_out = m_prev_out = init_output;
    m_counter = 0;
}

void SimDebounce::Reset()
{
    m_prev_out = m_init_out;
    m_counter = 0;
}

_BIT SimDebounce::Run(const _BIT input, const _UINT64 tick)
{
    if(input != m_prev_out)
    {
        ++m_counter;
        if((tick-m_counter) > m_timeout)
        {
            m_prev_out = input;
            m_counter = 0;
        }
    }
    else
    {
        m_counter = tick;
    }
    return m_prev_out;
}
////////////////////////////////////////////////////timer milli///////////////////////////////////////////////////////////////
void SimTimerMilli::Initialize(_UINT64 timeout, _BIT use_real_time)
{
    m_timeout = timeout;
    m_use_real_time = use_real_time;
    Reset();
}
void SimTimerMilli::Reset()
{
    if(m_use_real_time)
    {
#ifdef _BUILD_HAS_SIM_PNL
        m_offset = Sim_GetElapsedTime(0, 0)/1000000;
#elif defined(_BUILD_ARCH_FPGA)
        m_offset = millis();
#else
        m_offset = g_run_para.rt_clk_tick/1000;//using rt_clk_tick is more efficient
#endif
    }
    else
    {
        m_offset = 1;//to avoid >=
    }
    m_cur = m_offset;
}
_BIT SimTimerMilli::Run()
{
    if(m_use_real_time)
    {
#ifdef _BUILD_HAS_SIM_PNL
        m_cur = Sim_GetElapsedTime(0, 0)/1000000;
#elif defined(_BUILD_ARCH_FPGA)
        m_offset = millis();
#else
        m_cur = g_run_para.rt_clk_tick/1000;//using rt_clk_tick is more efficient
#endif
        if(m_cur - m_offset > m_timeout)
            return 1;
    }
    else
    {
        if(++m_cur > SIM_TICKS_PER_MSEC)
        {
            m_cur = 1;
            m_offset += SIM_TIME_STEP;
        }
        if(m_offset > m_timeout)
            return 1;
    }
    return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void SimTimerMicro::Initialize(_UINT64 timeout, _BIT use_real_time)
{
    m_timeout = timeout;
    m_use_real_time = use_real_time;
    Reset();
}
void SimTimerMicro::Reset()
{
    if(m_use_real_time)
    {
#ifdef _BUILD_HAS_SIM_PNL
        m_offset = Sim_GetElapsedTime(0, 0)/1000;
#elif defined(_BUILD_ARCH_FPGA)
        m_offset = micros();
#else
        m_offset = g_run_para.rt_clk_tick;//using rt_clk_tick is more efficient
#endif
    }
    else
    {
        m_offset = 1;//to avoid >=
    }
    m_cur = m_offset;
}
_BIT SimTimerMicro::Run()
{
    if(m_use_real_time)
    {
#ifdef _BUILD_HAS_SIM_PNL
        m_cur = Sim_GetElapsedTime(0, 0)/1000;
#elif defined(_BUILD_ARCH_FPGA)
        m_offset = micros();
#else
        m_cur = g_run_para.rt_clk_tick;//using rt_clk_tick is more efficient
#endif
        if(m_cur - m_offset > m_timeout)
            return 1;
    }
    else
    {
#if (SIM_TIME_RESOLUTION > 1000)
        m_offset += (SIM_TICKS_PER_USEC_INV*SIM_TIME_STEP);
        if(m_offset > m_timeout)
            return 1;
#else
        if(++m_cur > SIM_TICKS_PER_USEC)
        {
            m_cur = 1;
            m_offset += SIM_TIME_STEP;
        }
        if(m_offset > m_timeout)
            return 1;
#endif
    }
    return 0;
}
////////////////////////////////////////////////////////zcd///////////////////////////////////////////////////////////////////
void SimZcd::Initialize(_UINT64 cur_time)
{
    m_prev_out = 0;
    m_prev_amp = 0;
    m_prev_time = cur_time;
}

void SimZcd::Detect(_FXDPT_FLOAT in, _FXDPT_FLOAT ref_amp, _INT32 *out, _BYTE pos_polarity, _BYTE out_bipolar)
{
    //zcd means zero has to cross
    _INT32 oval = 0;
    if(m_prev_out == 0)
    {
        if(in >= ref_amp && pos_polarity == 1)//rising
        {
            oval = 1;
            m_prev_out = 1;
        }
        else if(in <= ref_amp && pos_polarity == 0)//falling
        {
            if(out_bipolar)
                oval = -1;
            else
                oval = 1;
            m_prev_out = 1;
        }
    }
    else
    {
        if(in < ref_amp && pos_polarity == 1)//rising. <= condition failing at low frequencies
            m_prev_out = 0;
        else if(in > ref_amp && pos_polarity == 0)//falling >= failing at low freqencies
            m_prev_out = 0;
    }
    *out = oval;
}

void SimZcd::Detect(_FXDPT_FLOAT in, _FXDPT_FLOAT ref_amp, _UINT64 cur_time, _INT32 *out, _UINT64 *zcd_time, _BYTE pos_polarity, _BYTE out_bipolar)
{
    //zcd means zero has to cross
    _INT32 oval = 0;
    if(m_prev_out == 0)
    {
        if(in >= ref_amp && pos_polarity == 1)//rising
        {
            oval = 1;
            m_prev_out = 1;
        }
        else if(in <= ref_amp && pos_polarity == 0)//falling
        {
            if(out_bipolar)
                oval = -1;
            else
                oval = 1;
            m_prev_out = 1;
        }
    }
    else
    {
        if(in < ref_amp && pos_polarity == 1)//rising. <= condition failing at low frequencies
            m_prev_out = 0;
        else if(in > ref_amp && pos_polarity == 0)//falling >= failing at low freqencies
            m_prev_out = 0;
    }
    _FXDPT_FLOAT diff_x2x1 = in - m_prev_amp;
    _FXDPT_FLOAT diff_xx1 = ref_amp - m_prev_amp;
    if(oval != 0)
    {
//        if(oval < 0)//may not be required
//        {
//            diff_x2x1 = m_prev_amp - in;
//            diff_xx1 = m_prev_amp - ref_amp;
//        }
#ifdef PREFER_FIXED_POINT_MATH
        _FXDPT_FLOAT diff_y2y1 = int_to_fxdpt((_INT64)(cur_time-m_prev_time));
        _FXDPT_FLOAT ztf = int_to_fxdpt((_INT64)m_prev_time);
        ztf += fxdpt_mul(fxdpt_div(diff_y2y1, diff_x2x1), diff_xx1);
        *zcd_time = fxdpt_to_int64(ztf);
#else
        _FXDPT_FLOAT diff_y2y1 = cur_time-m_prev_time;
        _FXDPT_FLOAT ztf = m_prev_time;
        ztf += ((diff_y2y1/diff_x2x1) * diff_xx1);
        *zcd_time = ztf;
#endif
    }
    m_prev_amp = in;
    m_prev_time = cur_time;
    *out = oval;
}
///////////////////////////////////////////////////////rms_avg_time/////////////////////////////////////////////////////////////
void SimRmsTime::Initialize(_UINT64 cur_time)
{
    m_zcd[0].Initialize(cur_time);
    m_zcd[1].Initialize(cur_time);
    m_sample_count = 1;
    m_sum = m_rms = 0;
}

_INT32 SimRmsTime::Run(_FXDPT_FLOAT in)
{
    _INT32 detected0 = 0, detected1 = 0;
    m_zcd[0].Detect(in, 0, &detected0, 1, 0);
    m_zcd[1].Detect(in, 0, &detected1, 0, 0);
    if(detected0 || detected1)
    {
#ifdef PREFER_FIXED_POINT_MATH
        m_rms = fxdpt_div(m_sum, m_sample_count);
#else
        m_rms = m_sum/(_FXDPT_FLOAT)m_sample_count;
#endif
        m_rms = SQRT(m_rms);
        m_sample_count = 1;
        m_sum = 0;
    }
    else
    {
        m_sum += SQUARE(in);
        ++m_sample_count;
    }
    if(detected1) detected0 = -1;
    return detected0;
}

_INT32 SimRmsTime::Run(_FXDPT_FLOAT in, _UINT64 cur_time, _UINT64 *zcd_time)
{
    _INT32 detected0 = 0, detected1 = 0;
    m_zcd[0].Detect(in, 0, cur_time, &detected0, zcd_time, 1, 0);
    m_zcd[1].Detect(in, 0, cur_time, &detected1, zcd_time, 0, 0);
    if(detected0 || detected1)
    {
#ifdef PREFER_FIXED_POINT_MATH
        m_rms = fxdpt_div(m_sum, m_sample_count);
#else
        m_rms = m_sum/(_FXDPT_FLOAT)m_sample_count;
#endif
        m_rms = SQRT(m_rms);
        m_sample_count = 1;
        m_sum = 0;
    }
    else
    {
        m_sum += SQUARE(in);
        ++m_sample_count;
    }
    if(detected1) detected0 = -1;
    return detected0;
}

void SimAvgTime::Initialize(_UINT64 cur_time)
{
    m_zcd[0].Initialize(cur_time);
    m_zcd[1].Initialize(cur_time);
    m_sample_count = 1;
    m_sum = m_avg = 0;
}

_INT32 SimAvgTime::Run(_FXDPT_FLOAT in)
{
    _INT32 detected0 = 0, detected1 = 0;
    m_zcd[0].Detect(in, 0, &detected0, 1, 0);
    m_zcd[1].Detect(in, 0, &detected1, 0, 0);
    if(detected0 || detected1)
    {
#ifdef PREFER_FIXED_POINT_MATH
        m_avg = fxdpt_div(m_sum, m_sample_count);
#else
        m_avg = m_sum/(_FXDPT_FLOAT)m_sample_count;
#endif
        m_sample_count = 1;
        m_sum = 0;
    }
    else
    {
        m_sum += in;
        ++m_sample_count;
    }
    if(detected1) detected0 = -1;
    return detected0;
}

_INT32 SimAvgTime::Run(_FXDPT_FLOAT in, _UINT64 cur_time, _UINT64 *zcd_time)
{
    _INT32 detected0 = 0, detected1 = 0;
    m_zcd[0].Detect(in, 0, cur_time, &detected0, zcd_time, 1, 0);
    m_zcd[1].Detect(in, 0, cur_time, &detected1, zcd_time, 0, 0);
    if(detected0 || detected1)
    {
#ifdef PREFER_FIXED_POINT_MATH
        m_avg = fxdpt_div(m_sum, m_sample_count);
#else
        m_avg = m_sum/(_FXDPT_FLOAT)m_sample_count;
#endif
        m_sample_count = 1;
        m_sum = 0;
    }
    else
    {
        m_sum += in;
        ++m_sample_count;
    }
    if(detected1) detected0 = -1;
    return detected0;
}
/////////////////////////////////////////////Blk Exec Interval///////////////////////////////////////////////////////////////
void SimBlkExecInterval::Initialize(_UINT32 offset, _UINT32 interval)
{
    m_offset = offset;
    m_interval = interval;
    m_count = 0;
}

_BOOLEAN SimBlkExecInterval::Run()
{
    _BOOLEAN ret = 0;
    if(m_offset > 0)
    {
        if(++m_count == m_offset)
        {
            m_offset = 0;
            m_count = 0;
            ret = 1;
        }
    }
    else
    {
        if(++m_count == m_interval)
        {
            m_count = 0;
            ret = 1;
        }
    }
    return ret;
}
/////////////////////////////////////////////Blk Exec Simulation Time///////////////////////////////////////////////////////////////
void SimBlkExecSimTime::Initialize(_UINT32 offset, _UINT64 interval, _BOOLEAN use_real_time)
{
    m_offset = offset;
    m_count = 0;
    m_time.Initialize(interval, use_real_time);
}

_BOOLEAN SimBlkExecSimTime::Run()
{
    _BOOLEAN ret = 0;
    if(m_offset > 0)
    {
        if(++m_count == m_offset)
        {
            m_offset = 0;
            m_count = 0;
            ret = 1;
            m_time.Reset();
        }
    }
    else
    {
        ret = m_time.Run();
        if(ret) m_time.Reset();
    }
    return ret;
}


///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/src_rp/thw/casp_thw_helper.cpp///////////////////////////////////////////////////////////////
/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "casp_thw_helper.h"
#include "sim_common_definitions.h"

void SimDigitalWrite::Initialize(_WORD pin, _BYTE val)
{
#ifndef _BUILD_ARCH_FPGA
    m_prev_val = val;
    digitalWrite(pin, val);
#endif
}

void SimDigitalWrite::Run(_WORD pin, _BYTE val)
{
#ifndef _BUILD_ARCH_FPGA
    if(val != m_prev_val)
    {
        m_prev_val = val;
        digitalWrite(pin, val);
    }
#endif
}

void SimAnalogWrite::Initialize(_WORD pin, int val)
{
#ifndef _BUILD_ARCH_FPGA
    m_prev_val = val;
    analogWrite(pin, val);
#endif
}

void SimAnalogWrite::Run(_WORD pin, int val)
{
#ifndef _BUILD_ARCH_FPGA
    if(val != m_prev_val)
    {
        m_prev_val = val;
        analogWrite(pin, val);
    }
#endif
}


///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/blocks/CASPBlocks/TargetHardware/TargetHw_IO/bin/TargetHw_IO863344046.cpp///////////////////////////////////////////////////////////////
#include "TargetHw_IO863344046.h"
#include "simcode.h"
#include "math.h"



///////////////////////////////////////////////////////////D:/CASP/deploy/CASP/blocks/CASPBlocks/General/UpDownCounter/bin/UpDownCounter107449447.cpp///////////////////////////////////////////////////////////////
/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "UpDownCounter107449447.h"
#include "math.h"
#include "sim_helper.h"
#include "simcode.h"

UpDownCounter107449447_i32::UpDownCounter107449447_i32(){
}

UpDownCounter107449447_i32::~UpDownCounter107449447_i32(){
}

void UpDownCounter107449447_i32::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
}

void UpDownCounter107449447_i32::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, _INT32 *in_up, _INT32 *in_down, _INT32 *out){
    if(count_on > 0)//rising edge
    {
        m_prev_up = input_low_threshold;
        m_prev_dn = input_low_threshold;
    }
    else
    {
        m_prev_up = input_high_threshold;
        m_prev_dn = input_high_threshold;
    }
    *out = init_value;
    m_init_count = 0;
}

void UpDownCounter107449447_i32::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _INT32 *in_up, _INT32 *in_down, _INT32 *out){
    const _INT32 up_val = *in_up;
    const _INT32 dn_val = *in_down;
    _INT32 out_val = *out;
    if(count_on > 0)//rising edge
    {
        if(m_prev_up <= input_low_threshold && up_val >= input_high_threshold)
            out_val += inc_dec_val;
        else if(m_prev_dn <= input_low_threshold && dn_val >= input_high_threshold)
            out_val -= inc_dec_val;
    }
    else
    {
        if(m_prev_up >= input_high_threshold && up_val <= input_low_threshold)
            out_val += inc_dec_val;
        if(m_prev_dn >= input_high_threshold && dn_val <= input_low_threshold)
            out_val -= inc_dec_val;
    }
    if(up_val <= input_low_threshold || up_val >= input_high_threshold)
        m_prev_up = up_val;
    if(dn_val <= input_low_threshold || dn_val >= input_high_threshold)
        m_prev_dn = dn_val;
    //check limits
    if(lower_limit != upper_limit)
    {
        if(out_val < lower_limit)
        {
            if(rst_on_limits)
                out_val = upper_limit;
            else
                out_val = lower_limit;
        }
        else if(out_val > upper_limit)
        {
            if(rst_on_limits)
                out_val = lower_limit;
            else
                out_val = upper_limit;
        }
    }
    if(m_init_count < 2)
        m_init_count++;
    else
        *out = out_val;
}

void UpDownCounter107449447_i32::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, _INT32 *in_up, _INT32 *in_down, _INT32 *out){
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UpDownCounter107449447_i64::UpDownCounter107449447_i64(){
}

UpDownCounter107449447_i64::~UpDownCounter107449447_i64(){
}

void UpDownCounter107449447_i64::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
}

void UpDownCounter107449447_i64::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, _INT64 *in_up, _INT64 *in_down, _INT32 *out){
    if(count_on > 0)//rising edge
    {
        m_prev_up = input_low_threshold;
        m_prev_dn = input_low_threshold;
    }
    else
    {
        m_prev_up = input_high_threshold;
        m_prev_dn = input_high_threshold;
    }
    *out = init_value;
    m_init_count = 0;
}

void UpDownCounter107449447_i64::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _INT64 *in_up, _INT64 *in_down, _INT32 *out){
    const _INT64 up_val = *in_up;
    const _INT64 dn_val = *in_down;
    _INT32 out_val = *out;
    if(count_on > 0)//rising edge
    {
        if(m_prev_up <= input_low_threshold && up_val >= input_high_threshold)
            out_val += inc_dec_val;
        else if(m_prev_dn <= input_low_threshold && dn_val >= input_high_threshold)
            out_val -= inc_dec_val;
    }
    else
    {
        if(m_prev_up >= input_high_threshold && up_val <= input_low_threshold)
            out_val += inc_dec_val;
        if(m_prev_dn >= input_high_threshold && dn_val <= input_low_threshold)
            out_val -= inc_dec_val;
    }
    if(up_val <= input_low_threshold || up_val >= input_high_threshold)
        m_prev_up = up_val;
    if(dn_val <= input_low_threshold || dn_val >= input_high_threshold)
        m_prev_dn = dn_val;
    //check limits
    if(lower_limit != upper_limit)
    {
        if(out_val < lower_limit)
        {
            if(rst_on_limits)
                out_val = upper_limit;
            else
                out_val = lower_limit;
        }
        else if(out_val > upper_limit)
        {
            if(rst_on_limits)
                out_val = lower_limit;
            else
                out_val = upper_limit;
        }
    }
    if(m_init_count < 2)
        m_init_count++;
    else
        *out = out_val;
}

void UpDownCounter107449447_i64::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, _INT64 *in_up, _INT64 *in_down, _INT32 *out){
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UpDownCounter107449447_r32::UpDownCounter107449447_r32(){
}

UpDownCounter107449447_r32::~UpDownCounter107449447_r32(){
}

void UpDownCounter107449447_r32::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
}

void UpDownCounter107449447_r32::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, _FXDPT_FLOAT32 *in_up, _FXDPT_FLOAT32 *in_down, _INT32 *out){
    if(count_on > 0)//rising edge
    {
        m_prev_up = input_low_threshold;
        m_prev_dn = input_low_threshold;
    }
    else
    {
        m_prev_up = input_high_threshold;
        m_prev_dn = input_high_threshold;
    }
    *out = init_value;
    m_init_count = 0;
}

void UpDownCounter107449447_r32::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _FXDPT_FLOAT32 *in_up, _FXDPT_FLOAT32 *in_down, _INT32 *out){
    const _FXDPT_FLOAT32 up_val = *in_up;
    const _FXDPT_FLOAT32 dn_val = *in_down;
    _INT32 out_val = *out;
    if(count_on > 0)//rising edge
    {
        if(m_prev_up <= input_low_threshold && up_val >= input_high_threshold)
            out_val += inc_dec_val;
        else if(m_prev_dn <= input_low_threshold && dn_val >= input_high_threshold)
            out_val -= inc_dec_val;
    }
    else
    {
        if(m_prev_up >= input_high_threshold && up_val <= input_low_threshold)
            out_val += inc_dec_val;
        if(m_prev_dn >= input_high_threshold && dn_val <= input_low_threshold)
            out_val -= inc_dec_val;
    }
    if(up_val <= input_low_threshold || up_val >= input_high_threshold)
        m_prev_up = up_val;
    if(dn_val <= input_low_threshold || dn_val >= input_high_threshold)
        m_prev_dn = dn_val;
    //check limits
    if(lower_limit != upper_limit)
    {
        if(out_val < lower_limit)
        {
            if(rst_on_limits)
                out_val = upper_limit;
            else
                out_val = lower_limit;
        }
        else if(out_val > upper_limit)
        {
            if(rst_on_limits)
                out_val = lower_limit;
            else
                out_val = upper_limit;
        }
    }
    if(m_init_count < 2)
        m_init_count++;
    else
        *out = out_val;
}

void UpDownCounter107449447_r32::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, _FXDPT_FLOAT32 *in_up, _FXDPT_FLOAT32 *in_down, _INT32 *out){
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UpDownCounter107449447_r64::UpDownCounter107449447_r64(){
}

UpDownCounter107449447_r64::~UpDownCounter107449447_r64(){
}

void UpDownCounter107449447_r64::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
}

void UpDownCounter107449447_r64::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, _FXDPT_FLOAT64 *in_up, _FXDPT_FLOAT64 *in_down, _INT32 *out){
    if(count_on > 0)//rising edge
    {
        m_prev_up = input_low_threshold;
        m_prev_dn = input_low_threshold;
    }
    else
    {
        m_prev_up = input_high_threshold;
        m_prev_dn = input_high_threshold;
    }
    *out = init_value;
    m_init_count = 0;
}

void UpDownCounter107449447_r64::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _FXDPT_FLOAT64 *in_up, _FXDPT_FLOAT64 *in_down, _INT32 *out){
    const _FXDPT_FLOAT64 up_val = *in_up;
    const _FXDPT_FLOAT64 dn_val = *in_down;
    _INT32 out_val = *out;
    if(count_on > 0)//rising edge
    {
        if(m_prev_up <= input_low_threshold && up_val >= input_high_threshold)
            out_val += inc_dec_val;
        else if(m_prev_dn <= input_low_threshold && dn_val >= input_high_threshold)
            out_val -= inc_dec_val;
    }
    else
    {
        if(m_prev_up >= input_high_threshold && up_val <= input_low_threshold)
            out_val += inc_dec_val;
        if(m_prev_dn >= input_high_threshold && dn_val <= input_low_threshold)
            out_val -= inc_dec_val;
    }
    if(up_val <= input_low_threshold || up_val >= input_high_threshold)
        m_prev_up = up_val;
    if(dn_val <= input_low_threshold || dn_val >= input_high_threshold)
        m_prev_dn = dn_val;
    //check limits
    if(lower_limit != upper_limit)
    {
        if(out_val < lower_limit)
        {
            if(rst_on_limits)
                out_val = upper_limit;
            else
                out_val = lower_limit;
        }
        else if(out_val > upper_limit)
        {
            if(rst_on_limits)
                out_val = lower_limit;
            else
                out_val = upper_limit;
        }
    }
    if(m_init_count < 2)
        m_init_count++;
    else
        *out = out_val;
}

void UpDownCounter107449447_r64::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, _FXDPT_FLOAT64 *in_up, _FXDPT_FLOAT64 *in_down, _INT32 *out){
}
