/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "sim_ipc.h"
//add additional header files above sim_ipc.h file

StaticSharedMem StaticShm;
StaticSharedMem::StaticSharedMem()
{
    for(_BYTE i=0; i<STATIC_SHM_SECS; ++i)
    {
        m_sec_idxa[i] = 0;
        m_sec_sizea[i] = 0;
    }
}

StaticSharedMem::~StaticSharedMem()
{
}

_BOOLEAN StaticSharedMem::pre_allocate(_BYTE index, _UINT32 size, _BIT write_only)
{
    if(index >= STATIC_SHM_SECS)
        return 0;
    m_sec_sizea[index] = size;
    return 1;
}

_BOOLEAN StaticSharedMem::allocate(_BYTE index, _BIT write_only)
{
    if(index >= STATIC_SHM_SECS)
        return 0;
    _UINT32 cur_idx = 0;
    for(_BYTE i=0; i<index; ++i)
    {
        if(cur_idx >= SIM_STATIC_SHM_SIZE)
            return 0;
        m_sec_idxa[i] = cur_idx;
        cur_idx += m_sec_sizea[i];
    }
    m_sec_idxa[index] = cur_idx;
    //init memory to zero
    for(_UINT32 i=0; i<m_sec_sizea[index]; ++i)
        m_shm_ref[i+cur_idx] = 0;
    return 1;
}

_UINT32 StaticSharedMem::write(_BYTE index, const _BYTE *data, _UINT32 size)
{
    volatile _BYTE *shm = GetMemPtr(index, size);
    if(shm == 0)
        return 0;
    for(_UINT32 i=0; i<size; ++i)
        shm[i] = data[i];
    return size;
}

_UINT32 StaticSharedMem::read(_BYTE index, _BYTE *buff, _UINT32 read_size)
{
    volatile _BYTE *shm = GetMemPtr(index, read_size);
    if(shm == 0)
        return 0;
    for(_UINT32 i=0; i<read_size; ++i)
        buff[i] = shm[i];
    return read_size;
}

volatile _BYTE* StaticSharedMem::GetMemPtr(_BYTE index, _UINT32 size)
{
    if(index >= STATIC_SHM_SECS || size < 1)
        return 0;
    _UINT32 idx = m_sec_idxa[index];
    if(idx+size > SIM_STATIC_SHM_SIZE)
        return 0;
    return &m_shm_ref[idx];
}
////////////////////////////////////////////////////////////HSEM////////////////////////////////////////////////////////////
HwdSemaphore HwdSem;
HwdSemaphore::HwdSemaphore()
{
}

HwdSemaphore::~HwdSemaphore()
{
}

_BOOLEAN HwdSemaphore::lock(const _UINT32 hsem_idx, _BOOLEAN blocking)
{
    if(hsem_idx >= HSEM_COUNT)
        return 0;
    _BYTE locked = m_hsem[hsem_idx];
    if(locked > 0 && blocking == false)
        return 0;
    while(m_hsem[hsem_idx]==1);
    m_hsem[hsem_idx] = 1;//lock
    return 1;
}

void HwdSemaphore::unlock(const _UINT32 hsem_idx)
{
    if(hsem_idx < HSEM_COUNT)
        m_hsem[hsem_idx] = 0;//unlock
}
