/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#ifndef XSOC_H
#define XSOC_H

#include "sim_generic_type_defs.h"
#include "xsoc_defs.h"
#include "xparameters.h"
#include "sim_debug.h"

#ifdef __cplusplus
extern "C"{
#endif

#define HIGH                    1
#define LOW                     0
#define INPUT                   0
#define OUTPUT                  1
#define INPUT_PULLUP            2
#define INPUT_PULLDOWN          3
#define OUTPUT_OPENDRAIN        4
#define INPUT_DISABLE           5
#define LSBFIRST                0
#define MSBFIRST                1
#define CHANGE                  4
#define FALLING                 2
#define RISING                  3

//
#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))
#define constrain(amt,low,high) ((amt)<(low)?(low):((amt)>(high)?(high):(amt)))
#define radians(deg) ((deg)*DEG_TO_RAD)
#define degrees(rad) ((rad)*RAD_TO_DEG)
#define sq(x) ((x)*(x))

#define lowByte(w) ((_BYTE) ((w) & 0xff))
#define highByte(w) ((_BYTE) ((w) >> 8))

#define bitRead(value, bit) (((value) >> (bit)) & 0x01)
#define bitSet(value, bit) ((value) |= (1UL << (bit)))
#define bitClear(value, bit) ((value) &= ~(1UL << (bit)))
#define bitWrite(value, bit, bitvalue) (bitvalue ? bitSet(value, bit) : bitClear(value, bit))

#define sev() __asm__("sev")

//user functions
void setup();
void loop();
void analogReadResolution(unsigned char);//sets ADC resolution, not used
void analogWriteResolution(unsigned char);//sets resolution for all PWM outputs
void analogReadResolutionPin(unsigned short res, unsigned char pin);
void analogWriteResolutionPin(unsigned short res, unsigned char pin);
void pinMode(unsigned char, unsigned char);
void digitalWrite(unsigned char, unsigned char);
int digitalRead(unsigned char);
int analogRead(unsigned char);
//void analogReference(unsigned char mode);
void analogWrite(unsigned char, int);
unsigned long long millis();
unsigned long long micros();
void delay(unsigned long);
void delayMicroseconds(unsigned long);
void attachInterrupt(void (*function)(void), unsigned int mode, unsigned int pullup, int pin_idx);
//internal functions
void init1();
void init2();
void initVariant1();//executed before setup()
void initVariant2();//executed after setup(). useful in adc/pwm resolution settings
void loop_system(void);
//
void initADC();
void startADC();
void updateADC();
void initPWM();//initializes pwm & dac
void startPWM();
void initDIO();
///////////////////////////serial ports///////////////////////////////////
#define SERIAL_TX_BUFFER_SIZE	64
#define SERIAL_RX_BUFFER_SIZE	64
//
#define SERIAL_8N1		0x01
#define SERIAL_8E1		0x02
#define SERIAL_8O1		0x03
//
class SerialComm
{
public:
    SerialComm();
    ~SerialComm();
    void begin(unsigned int, _BYTE);
    void begin(unsigned long baud) { begin(baud, SERIAL_8N1); }
    void end();
    int available();
    int peek(int index = 0);
    int read();
    int readBytes(char*, int);
    inline int readBytes(unsigned char *buffer, int length) {return readBytes((char*)buffer, length);}
    void write(unsigned char);
    int write(unsigned char*, int);
    void flush(bool wait=true);
private:
    volatile int m_read_index;
    _BYTE m_status;//0-for no errors
};
extern SerialComm Serial;
/////////////////////////////firmware helpers///////////////////////////////
////////////////////////////////////////////////////////////////////////////
//This function is executed in case of error occurrence
void _Error_Handler(const char *, int);
//#define Error_Handler() _Error_Handler(__FILE__, __LINE__)
////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
}
#endif
#endif //XSOC_H
