/************************CASP_COPYRIGHT**********************************
** Copyright (c) AadhunikLabs. All rights reserved.
** This file is part of CASP.
** Licensees holding valid licenses may use this file in accordance
** with the license agreement provided with the software.
*************************************************************************/
#include "xsoc.h"
#include "sim_helper.h"
#include "xuartps.h"
#include "xil_exception.h"
#include "xplatform_info.h"
#ifdef XPAR_INTC_0_DEVICE_ID
#include "xintc.h"
#else
#include "xscugic.h"
#endif
//
SerialComm Serial;
unsigned char g_serial_rxbuffer[SERIAL_RX_BUFFER_SIZE];
unsigned char g_serial_txbuffer[SERIAL_TX_BUFFER_SIZE];
volatile int g_serial_rx_size, g_serial_tx_size;
//
#ifdef XPAR_INTC_0_DEVICE_ID
#define INTC                XIntc
#define UART_DEVICE_ID		XPAR_XUARTPS_0_DEVICE_ID
#define INTC_DEVICE_ID		XPAR_INTC_0_DEVICE_ID
#define UART_INT_IRQ_ID		XPAR_INTC_0_UARTPS_0_VEC_ID
#else
#define INTC                XScuGic
#define UART_DEVICE_ID		XPAR_XUARTPS_0_DEVICE_ID
#define INTC_DEVICE_ID		XPAR_SCUGIC_SINGLE_DEVICE_ID
#define UART_INT_IRQ_ID		XPAR_XUARTPS_1_INTR
#endif
XUartPs g_uart0_ps;
//INTC g_uart0_interruptController;
//int SetupInterruptSystem(INTC *IntcInstancePtr, XUartPs *UartInstancePtr, u16 UartIntrId);
//void UART0_INTR_Handler(void *CallBackRef, u32 Event, unsigned int EventData);
//
SerialComm::SerialComm()
{
    m_status = XST_FAILURE;
}
SerialComm::~SerialComm()
{
}
void SerialComm::begin(unsigned int baud, _BYTE type)
{
    m_status = XST_FAILURE;
    //Initialize the UART driver so that it's ready to use Look up the configuration in the config table and then initialize it.
    XUartPs_Config *config = XUartPs_LookupConfig(UART_DEVICE_ID);
    if(config == NULL)
        return;
    m_status = XUartPs_CfgInitialize(&g_uart0_ps, config, config->BaseAddress);
    if(m_status != XST_SUCCESS)
        return;
    //set basic parameters
    XUartPsFormat format;
    format.BaudRate = baud;
    format.DataBits = 8;
    format.StopBits = 1;
    if(type == SERIAL_8N1) format.Parity = XUARTPS_FORMAT_NO_PARITY;
    else if(type == SERIAL_8E1) format.Parity = XUARTPS_FORMAT_EVEN_PARITY;
    else if(type == SERIAL_8O1) format.Parity = XUARTPS_FORMAT_ODD_PARITY;
    XUartPs_SetDataFormat(&g_uart0_ps, &format);
    //Connect the UART to the interrupt subsystem such that interrupts can occur. This function is application specific.
    //m_status = SetupInterruptSystem(&g_uart0_interruptController, &g_uart0_ps, UART_INT_IRQ_ID);
    //if (m_status != XST_SUCCESS)
    //    return;
    //Setup the handlers for the UART that will be called from the interrupt context when data has been sent and received, specify
    //a pointer to the UART driver instance as the callback reference so the handlers are able to access the instance data
    //XUartPs_SetHandler((XUartPs*)&g_uart0_interruptController, (XUartPs_Handler)UART0_INTR_Handler, &g_uart0_ps);
    //Enable the interrupt of the UART so interrupts will occur
    //u32 IntrMask =
    //    XUARTPS_IXR_TOUT | XUARTPS_IXR_PARITY | XUARTPS_IXR_FRAMING |
    //    XUARTPS_IXR_OVER | XUARTPS_IXR_TXEMPTY | XUARTPS_IXR_RXFULL |
    //    XUARTPS_IXR_RXOVR;
    //if(g_uart0_ps.Platform == XPLAT_ZYNQ_ULTRA_MP) IntrMask |= XUARTPS_IXR_RBRK;
    //XUartPs_SetInterruptMask(&g_uart0_ps, IntrMask);
    //Set the receiver timeout. If it is not set, and the last few bytes of data do not trigger the over-water or full interrupt, the bytes
    //will not be received. By default it is disabled. The setting of 8 will timeout after 8 x 4 = 32 character times.
    //Increase the time out value if baud rate is high, decrease it if baud rate is low.
    //XUartPs_SetRecvTimeout(&g_uart0_ps, 1);
    //
    g_serial_rx_size = g_serial_tx_size = 0;
    m_read_index = 0;//first read/write then increment
}
void SerialComm::end()
{
}
int SerialComm::available()
{
    g_serial_rx_size += XUartPs_Recv(&g_uart0_ps, &g_serial_rxbuffer[g_serial_rx_size], SERIAL_RX_BUFFER_SIZE-g_serial_rx_size);
    return g_serial_rx_size-m_read_index;
}
int SerialComm::peek(int index)
{
    available();
    if(index >= g_serial_rx_size)
        return -1;
    return (int)g_serial_rxbuffer[index];
}
int SerialComm::read()
{
    available();
    if(m_read_index >= g_serial_rx_size || g_serial_rx_size < 1)
        return -1;
    int data = (int)g_serial_rxbuffer[m_read_index];
    ++m_read_index;
    if(m_read_index == g_serial_rx_size)
    {
        m_read_index = 0;
         g_serial_rx_size = 0;
    }
    return data;
}
int SerialComm::readBytes(char *buffer, int buffer_size)
{
    available();
    if(g_serial_rx_size < 1 || buffer_size < 1)
        return 0;
    int buff_idx = 0;
    while(m_read_index < g_serial_rx_size && buff_idx < buffer_size)
    {
        buffer[buff_idx] = (char)g_serial_rxbuffer[m_read_index];
        ++m_read_index;
        ++buff_idx;
    }
    if(m_read_index == g_serial_rx_size)
    {
        m_read_index = 0;
        g_serial_rx_size = 0;
    }
    return buff_idx;
}
int SerialComm::write(unsigned char *data, int size)
{
    for(int i=0; i<size; ++i)
        write(data[i]);
    if(g_serial_tx_size < SERIAL_TX_BUFFER_SIZE)//else condition not required. taken care in called function
        flush(false);
    return size;
}
void SerialComm::write(unsigned char data)
{
    if(g_serial_tx_size == SERIAL_TX_BUFFER_SIZE)
        flush(true);
    g_serial_txbuffer[g_serial_tx_size] = (_BYTE)data;
    ++g_serial_tx_size;
}
void SerialComm::flush(bool wait)
{
    XUartPs_Send(&g_uart0_ps, g_serial_txbuffer, g_serial_tx_size);
    while(XUartPs_IsSending(&g_uart0_ps) && wait);
    g_serial_tx_size = 0;
}
/*
int SetupInterruptSystem(INTC *IntcInstancePtr, XUartPs *UartInstancePtr, u16 UartIntrId)
{
    int Status;
#ifdef XPAR_INTC_0_DEVICE_ID
#ifndef TESTAPP_GEN
    //Initialize the interrupt controller driver so that it's ready to use.
    Status = XIntc_Initialize(IntcInstancePtr, INTC_DEVICE_ID);
    if (Status != XST_SUCCESS) {
        return XST_FAILURE;
    }
#endif
    //Connect the handler that will be called when an interrupt for the device occurs, the handler defined above performs the specific interrupt processing for the device.
    Status = XIntc_Connect(IntcInstancePtr, UartIntrId,
        (XInterruptHandler) XUartPs_InterruptHandler, UartInstancePtr);
    if (Status != XST_SUCCESS) {
        return XST_FAILURE;
    }

#ifndef TESTAPP_GEN
    //Start the interrupt controller so interrupts are enabled for all devices that cause interrupts.
    Status = XIntc_Start(IntcInstancePtr, XIN_REAL_MODE);
    if (Status != XST_SUCCESS) {
        return XST_FAILURE;
    }
#endif
    //Enable the interrupt for uart
    XIntc_Enable(IntcInstancePtr, UartIntrId);

    #ifndef TESTAPP_GEN
    //Initialize the exception table.
    Xil_ExceptionInit();

    //Register the interrupt controller handler with the exception table.
    Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
                (Xil_ExceptionHandler) XIntc_InterruptHandler,
                IntcInstancePtr);
    #endif
#else
#ifndef TESTAPP_GEN
    XScuGic_Config *IntcConfig; //Config for interrupt controller

    //Initialize the interrupt controller driver
    IntcConfig = XScuGic_LookupConfig(INTC_DEVICE_ID);
    if (NULL == IntcConfig) {
        return XST_FAILURE;
    }

    Status = XScuGic_CfgInitialize(IntcInstancePtr, IntcConfig,
                    IntcConfig->CpuBaseAddress);
    if (Status != XST_SUCCESS) {
        return XST_FAILURE;
    }

    //Connect the interrupt controller interrupt handler to the hardware interrupt handling logic in the processor.
    Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT, (Xil_ExceptionHandler) XScuGic_InterruptHandler, IntcInstancePtr);
#endif

    //Connect a device driver handler that will be called when an interrupt for the device occurs, the device driver handlers performs the specific interrupt processing for the device
    Status = XScuGic_Connect(IntcInstancePtr, UartIntrId, (Xil_ExceptionHandler) XUartPs_InterruptHandler, (void *) UartInstancePtr);
    if (Status != XST_SUCCESS) {
        return XST_FAILURE;
    }

    //Enable the interrupt for the device
    XScuGic_Enable(IntcInstancePtr, UartIntrId);

#endif
#ifndef TESTAPP_GEN
    //Enable interrupts
     Xil_ExceptionEnable();
#endif

    return XST_SUCCESS;
}

void UART0_INTR_Handler(void *CallBackRef, u32 Event, unsigned int EventData)
{
    //All of the data has been sent
    //if(Event == XUARTPS_EVENT_SENT_DATA) {TotalSentCount = EventData;}

    //All of the data has been received
    //if(Event == XUARTPS_EVENT_RECV_DATA){g_serial_rx_size = EventData;}

    //Data was received, but not the expected number of bytes, a timeout just indicates the data stopped for 8 character times
    //if(Event == XUARTPS_EVENT_RECV_TOUT){g_serial_rx_size = EventData;}

    //Data was received with an error, keep the data but determine what kind of errors occurred
    //if(Event == XUARTPS_EVENT_RECV_ERROR){TotalReceivedCount = EventData; TotalErrorCount++;}

    //Data was received with an parity or frame or break error, keep the data but determine what kind of errors occurred. Specific to Zynq Ultrascale+ MP.
    //if(Event == XUARTPS_EVENT_PARE_FRAME_BRKE){TotalReceivedCount = EventData; TotalErrorCount++;}

    //Data was received with an overrun error, keep the data but determine what kind of errors occurred. Specific to Zynq Ultrascale+ MP.
    //if(Event == XUARTPS_EVENT_RECV_ORERR){TotalReceivedCount = EventData; TotalErrorCount++;}
}
*/
